#coding:utf-8
"""
Created on 27.02.2010

@author: prefer
"""
import os
import json

from django.conf import settings
from django.utils.html import escapejs

from base import BaseExtField

from m3 import actions
from m3_ext.ui.misc import ExtJsonStore
from m3_ext.ui.fields.base import BaseExtTriggerField
from m3_ext.ui.base import BaseExtComponent, ExtUIComponent
from m3.actions import ControllerCache
from m3.actions.interfaces import ISelectablePack, IMultiSelectablePack


#==============================================================================
class ExtDictSelectField(BaseExtTriggerField):
    """
    Поле с выбором из справочника
    """
    class ExtTrigger(BaseExtComponent):
        def __init__(self, *args, **kwargs):
            self.icon_cls = None
            self.handler = None
            self.init_component(*args, **kwargs)

        def render(self):
            res = 'iconCls: "%s"' % (self.icon_cls if self.icon_cls else '')
            res += ',handler: %s' % self.handler if self.handler else ''
            return res

    def __init__(self, *args, **kwargs):
        super(ExtDictSelectField, self).__init__(*args, **kwargs)

        # Эти атрибуты отвечают за отображение кнопок действий в строке выбора:
        # Выпадающий список
        self.hide_trigger = True
        # Очистка поля
        self.hide_clear_trigger = False
        # Редактирование выбранного элемента
        self.hide_edit_trigger = False
        # Выбора из справочника
        self.hide_dict_select_trigger = False

        # количество знаков, с которых начинаются запросы на autocomplete
        self.min_chars = 2

        self.set_store(ExtJsonStore())

        #self.width = 150
        self.default_text = None

        self.ask_before_deleting = True

        self.url = None
        self.edit_url = None
        self.autocomplete_url = None

        # это взято из магического метода configure_edit_field
        # из mis.users.forms
        self.value_field = 'id'
        # и это тоже взято оттуда же
        self.query_param = 'filter'
        # по умолчанию отображаем значение поля name
        self.display_field = 'name'

        # Из-за ошибки убраны свойства по умолчанию
        self.total = 'total'
        self.root = 'rows'

        # значение, которое будет передано в store
        self.__record_value = {}

        self._triggers = []

        self._pack = None

        self.init_component(*args, **kwargs)

        # внутренние переменные
        self.__action_select = None
        self.__action_data = None

    @property
    def handler_afterselect(self):
        return self._listeners.get('afterselect')

    @handler_afterselect.setter
    def handler_afterselect(self, function):
        self._listeners['afterselect'] = function

    @property
    def handler_beforerequest(self):
        return self._listeners.get('beforerequest')

    @handler_beforerequest.setter
    def handler_beforerequest(self, function):
        self._listeners['beforerequest'] = function

    @property
    def handler_changed(self):
        return self._listeners.get('changed')

    @handler_changed.setter
    def handler_changed(self, function):
        self._listeners['changed'] = function

    #==========================================================================
    # Экшены для управления процессом работы справочника
    #==========================================================================

    #==========================================================================
    #  Получение окна выбора значения
    def _get_action_select(self):
        return self.__action_autocomplete

    def _set_action_select(self, value):
        self.__action_autocomplete = value
        if isinstance(value, actions.Action):
            self.autocomplete_url = value.get_absolute_url()
    action_select = property(
        _get_action_select,
        _set_action_select,
        doc=(
            'Действие, которое используется '
            'для получения окна выбора значения'
        ))
    #==========================================================================

    #==========================================================================
    # Получение списка для получения списка значений
    # (используется в автозаполнении)
    def _get_action_data(self):
        return self.__action_data

    def _set_action_data(self):
        return self.__action_data
    action_data = property(
        _get_action_data,
        _set_action_data,
        doc=(
            'Действие для получения списка '
            'строковых значений для '
        ))
    #==========================================================================

    @property
    def url(self):
        return self.__url

    @url.setter
    def url(self, value):
        self.__url = value

    @property
    def edit_url(self):
        return self.__edit_url

    @edit_url.setter
    def edit_url(self, value):
        self.__edit_url = value

    @property
    def autocomplete_url(self):
        return self.__autocomplete_url

    @autocomplete_url.setter
    def autocomplete_url(self, value):
        if value:
            self.editable = True
            self.get_store().url = value
        self.__autocomplete_url = value

    @property
    def record_value(self):
        return self.__record_value

    @record_value.setter
    def record_value(self, val):
        self.__record_value = val

    @property
    def value(self):
        return self.__value

    @value.setter
    def value(self, val):
        self.__value = val

    def configure_by_dictpack(self, pack, controller=None):
        """
        Метод настройки поля выбора из справочника на основе
        переданного ActionPack работы со справочниками.
        @param pack: Имя класса или класс пака.
        @controller: Контроллер в котором будет искаться пак.
        Если не задан, то ищем во всех.
        @deprecated: 0.4
        """
        registered_pack = ControllerCache.find_pack(pack)
        if not registered_pack:
            raise Exception(
                'Pack %s not found!' % pack
            )
        self.url = registered_pack.get_select_url()
        self.autocomplete_url = registered_pack.rows_action.get_absolute_url()
        # TODO: можно ли обойтись без bind_back?
        self.bind_pack = registered_pack

    def set_value_from_model(self, obj):
        """
        Устанавливает значения value и default_text по экземпляру модели obj.
        Причем они могут быть методами, например обернутыми json_encode.
        Это позволяет избежать двойного присваивания в коде.
        """

        value = getattr(obj, self.value_field)
        self.value = escapejs(value() if callable(value) else value)

        value = getattr(obj, self.display_field)
        self.default_text = value() if callable(value) else value

        # также запомним объект как значение
        # будем брать только те атрибуты, которые есть в fields
        attr_set = set(self.fields)
        attr_set.add(self.value_field)
        attr_set.add(self.display_field)
        self.__record_value = {}
        for attr in attr_set:
            value = getattr(obj, attr, None)
            self.__record_value[attr] = value() if callable(value) else value

    @property
    def pack(self):
        return self._pack

    @pack.setter
    def pack(self, ppack):
        self._set_urls_from_pack(ppack)

    @property
    def total(self):
        return self.get_store().total_property

    @total.setter
    def total(self, value):
        self.get_store().total_property = value

    @property
    def root(self):
        return self.get_store().root

    @root.setter
    def root(self, value):
        self.get_store().root = value

    def add_trigger(self, *args, **kwargs):
        self._triggers.append(ExtDictSelectField.ExtTrigger(*args, **kwargs))

    def t_render_triggers(self):
        return '[%s]' % ','.join(
            ['{%s}' % item.render() for item in self._triggers])

    def _set_urls_from_pack(self, ppack):
        """
        Настраивает поле выбора под указанный экшенпак ppack.
        Причем в качестве аргумента может быть как сам класс пака,
        так и имя. Это связано с тем, что не во всех формах можно
        импортировать паки и может произойти кроссимпорт.
        Поиск пака производится по всем экшенконтроллерам в системе.
        Используется первый найденный, т.к. при правильном дизайне
        один и тот же пак не должен быть в нескольких
        контроллерах одновременно.
        @param ppack: Имя класса пака или класс пака.
        """
        assert isinstance(ppack, basestring) or hasattr(ppack, '__bases__'), (
            'Argument %s must be a basestring or class' % ppack)
        ppack = ControllerCache.find_pack(ppack)
        assert ppack, 'Pack %s not found in ControllerCache' % ppack
        assert isinstance(ppack, ISelectablePack), (
            'Pack %s must provide ISelectablePack interface' % ppack)
        self._pack = ppack

        # старый спосом подключения Pack теперь не действует
        # - всё должно быть в рамках интерфейса ISelectablePack

        # url формы редактирования элемента
        self.edit_url = ppack.get_edit_url()
        # url автокомплита и данных
        self.autocomplete_url = ppack.get_autocomplete_url()
        # url формы выбора
        self.url = ppack.get_select_url()

    def render_params(self):
        action_context = None
        # FIXME: Почему то нет в конструкторе.
        if self.action_context:
            # функция
            action_context = self.action_context.json

        for args in (
            ('askBeforeDeleting', self.ask_before_deleting),
            ('actions', {
                'actionSelectUrl': self.url,
                'actionEditUrl': self.edit_url,
                'contextJson':  action_context}),
            ('defaultText', self.default_text),
            ('hideClearTrigger', self.hide_clear_trigger),
            ('hideEditTrigger', self.hide_edit_trigger),
            ('hideDictSelectTrigger', self.hide_dict_select_trigger),
            ('defaultValue', self.value),
            ('customTriggers', self.t_render_triggers, self._triggers),
            ('recordValue', json.dumps(self.record_value), self.record_value),
        ):
            self._put_params_value(*args)

    def render(self):
        self.render_base_config()
        self.render_params()

        base_config = self._get_config_str()
        params = self._get_params_str()
        return 'createAdvancedComboBox({%s},{%s})' % (base_config, params)


#==============================================================================
class ExtSearchField(BaseExtField):
    """Поле поиска"""
    def __init__(self, *args, **kwargs):
        super(ExtSearchField, self).__init__(*args, **kwargs)
        self.query_param = None
        self.empty_text = None
        self.component_for_search = None
        self.init_component(*args, **kwargs)

    def render_base_config(self):
        super(ExtSearchField, self).render_base_config()
        self._put_params_value('paramName', self.query_param)
        self._put_params_value('emptyText', self.empty_text)

    def render(self):
        assert isinstance(self.component_for_search, ExtUIComponent)
        self.render_base_config()
        base_config = self._get_config_str()
        # Строка рендера как в шаблоне
        return (
            'new Ext.app.form.SearchField({%s, '
            'getComponentForSearch: function(){return Ext.getCmp("%s");}})'
        ) % (base_config, self.component_for_search.client_id)


#==============================================================================
class ExtFileUploadField(BaseExtField):
    """
    Компонент загрузки файлов на сервер.
    """

    # Префикс добавляется к скрытому полю, где передается файл
    PREFIX = 'file_'

    def __init__(self, *args, **kwargs):
        super(ExtFileUploadField, self).__init__(*args, **kwargs)
        self.file_url = None

        # Пример использования:
        # possible_file_extensions = ('png', 'jpeg', 'gif', 'bmp')

        #Пусто
        self.possible_file_extensions = ()
        self.init_component(*args, **kwargs)

        # Привязка к файлу
        self._memory_file = None

    def render_possible_file_extensions(self):
        p = self.possible_file_extensions
        assert isinstance(p, (basestring, list, tuple)), (
            u'File extensions argument must be '
            u'type of basestring, tuple or list'
        )
        return ','.join(p) if not isinstance(p, basestring) else p

    def render_params(self):
        super(ExtFileUploadField, self).render_params()
        self._put_params_value('prefixUploadField', ExtFileUploadField.PREFIX)
        self._put_params_value('fileUrl', self.file_url)
        self._put_params_value(
            'possibleFileExtensions', self.render_possible_file_extensions())

    def render(self):
        self.render_base_config()
        self.render_params()
        base_config = self._get_config_str()
        params_config = self._get_params_str()
        return 'new Ext.ux.form.FileUploadField({%s}, {%s})' % (
            base_config, params_config)

    @property
    def memory_file(self):
        return self._memory_file

    @memory_file.setter
    def memory_file(self, memory_file):
        self._memory_file = memory_file


#==============================================================================
class ExtImageUploadField(ExtFileUploadField):
    """
    Компонент загрузки изображений
    """
    MAX = 'max'
    MIN = 'min'
    MIDDLE = 'middle'
    THUMBNAIL_PREFIX = 'thumbnail_'
    MIN_THUMBNAIL_PREFIX = '%s_%s' % (MIN, THUMBNAIL_PREFIX)
    MIDDLE_THUMBNAIL_PREFIX = '%s_%s' % (MIDDLE, THUMBNAIL_PREFIX)
    MAX_THUMBNAIL_PREFIX = '%s_%s' % (MAX, THUMBNAIL_PREFIX)

    def __init__(self, *args, **kwargs):

        self.middle_thumbnail_size = (
            self.max_thumbnail_size) = self.min_thumbnail_size = None

        self.thumbnail_size = (300, 300)

        # Использовать ли миниатюры для изображений
        self.thumbnail = True

        # Высота и ширина изображения. Изображение будет подгоняться под
        # эту высоту
        self.image_max_size = (600, 600)

        super(ExtImageUploadField, self).__init__(*args, **kwargs)

        # Умолчательный параметр, иначе контрол разъедется
        self.width = 300
        # начальные допустимые расширения
        self.possible_file_extensions = ('png', 'jpeg', 'gif', 'bmp', 'jpg')

        self.init_component(*args, **kwargs)

    @property
    def thumbnail_size(self):
        return self.min_thumbnail_size

    @thumbnail_size.setter
    def thumbnail_size(self, value):
        self.min_thumbnail_size = value

    def render_params(self):
        super(ExtImageUploadField, self).render_params()
        self._put_params_value('thumbnail', self.thumbnail)
        if self.thumbnail:
            assert isinstance(self.thumbnail_size, tuple) and len(
                self.thumbnail_size) == 2
            self._put_params_value(
                'thumbnailWidth', self.min_thumbnail_size[0],
                self.thumbnail
            )
            self._put_params_value(
                'thumbnailHeight', self.min_thumbnail_size[1],
                self.thumbnail
            )
            self._put_params_value(
                'prefixThumbnailImg',
                ExtImageUploadField.MIN_THUMBNAIL_PREFIX,
                self.thumbnail
            )
            self._put_params_value('thumbnail', self.thumbnail)

    def render(self):
        self.render_base_config()
        self.render_params()
        base_config = self._get_config_str()
        params_config = self._get_params_str()
        return 'new Ext.ux.form.ImageUploadField({%s}, {%s})' % (
            base_config, params_config)

    @staticmethod
    def _prefix_by_type(type_img=None):
        if type_img == ExtImageUploadField.MIDDLE:
            return ExtImageUploadField.MIDDLE_THUMBNAIL_PREFIX
        elif type_img == ExtImageUploadField.MAX:
            return ExtImageUploadField.MAX_THUMBNAIL_PREFIX
        else:
            return ExtImageUploadField.MIN_THUMBNAIL_PREFIX

    @staticmethod
    def get_thumbnail_path(path, size=None):
        if os.path.exists(path):
            _dir = os.path.dirname(path)
            name = os.path.basename(path)
            prefix = ExtImageUploadField._prefix_by_type(size)
            return os.path.join(_dir, prefix + name)

    @staticmethod
    def get_thumbnail_url(name, type_img=None):
        """
        Возвращает url до thumbnail
        @param name: Имя
        @param size: Размер
        """
        base_url, file_name = os.path.split(name)
        prefix = ExtImageUploadField._prefix_by_type(type_img)
        return (
            '%s/%s' % (
                settings.MEDIA_URL,
                '%s/%s%s' % (
                    base_url,
                    prefix,
                    file_name
                ))).replace('//', '/')

    @staticmethod
    def get_image_url(name):
        return '%s/%s' % (settings.MEDIA_URL, name)


#==============================================================================
class ExtMultiSelectField(ExtDictSelectField):
    """
    Множественный выбор из справочника.
    Может использоваться также как стандартный ExtDictSelectField,
    При использовании следует обратить внимание,
    что биндиться к полям формы этот компонент не будет,
    тк в качестве value принимается список чего-либо,
    откуда на клиенте будут извлекаться объекты по value_field и display_field.
    Не рекомендуются передавать в value список моделей,
    тк все что передано будет преобразовано в json строку для
    отдачи на клиент, а от полной сериализации моделей хорошего мало.
    Лушче отдавать список словарей.

    Кроме того можно использовать как локальный комбокс с галочками,
    для этого достаточно задать Store методом
    set_store и в value, если нужно, передать список со значениями value_field.

    """
    def __init__(self, *args, **kwargs):
        self.delimeter = ','
        self.multiple_display_value = None
        self._value = ''
        self._init_flag = True

        super(ExtMultiSelectField, self).__init__(*args, **kwargs)
        self.init_component(*args, **kwargs)
        self.hidden_name = self.client_id

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        if self._init_flag:
            self._init_flag = False
            return

        if not value:
            value = []

        if isinstance(value, basestring):
            value = json.loads(value)

        if isinstance(value, (list, tuple)):
            self._value = json.dumps(value)
        else:
            raise TypeError(
                u'ExtMultiSelectField value must be list or tuple of values')

    @property
    def pack(self):
        return self._pack

    @pack.setter
    def pack(self, ppack):
        assert isinstance(ppack, basestring) or hasattr(ppack, '__bases__'), (
            'Argument %s must be a basestring or class' % ppack)
        ppack_class = ControllerCache.find_pack(ppack)
        assert isinstance(ppack_class, IMultiSelectablePack), (
            'Pack %s must provide IMultiSelectablePack interface' % ppack)
        self._set_urls_from_pack(ppack)
        self.url = self._pack.get_multi_select_url()

    def render(self):
        self.render_base_config()
        self.render_params()

        base_config = self._get_config_str()
        params = self._get_params_str()
        return 'new Ext.m3.MultiSelectField({%s}, {%s})' % (
            base_config, params)

    def render_base_config(self):
        self.pre_render()

        super(ExtMultiSelectField, self).render_base_config()
        self._put_config_value('delimeter', self.delimeter)
        if self.multiple_display_value:
            self._put_config_value(
                'multipleDisplayValue', self.multiple_display_value)
        self._put_config_value('delimeter', self.delimeter)



#coding:utf-8
"""
Модуль шорткатов для подсистемы m3_ext_demo.ui

Created on 24.03.2010

@author: akvarats
"""
import json

from django import http

from m3_ext.ui.containers.forms import ExtForm
from m3_ext.ui.windows.base import BaseExtWindow


def js_submit_form(
        form,
        success_handler='',
        failure_handler='',
        invalid_handler='',
        params=None):
    """
    :deprecated: Использовать методы окна, например,
        в ExtEditWindow метод submitForm

    Шорткат, который позволяет назначить хендлер для обработки субмита формы
    :param form: экземпляр ExtForm
    :param success_handler:
        анонимная JS функция срабатывающая при успешном исходе субмита
    :param failure_handler:
        анонимная JS функция срабатывающая ошибке
    :param invalid_handler:
        JS действия при отказе сабмита из-за неверных значений полей
    :param params: словарь с доп. параметрами передаваемыми через POST
    """
    assert isinstance(form, ExtForm)
    template = u'''
function(){
    var form = Ext.getCmp('%(form_id)s').getForm();
    if (!form.isValid()){
        %(invalid_handler)s
        return;
    }
    form.submit({
        %(submit_params)s
    });
}
'''
    # Эти функции смотри в static/js/common.js
    failure_handler = failure_handler or 'uiAjaxFailMessage'
    submit_params = []
    if form.url:
        submit_params.append("url: '%s'" % form.url)
    if success_handler:
        submit_params.append("success: " + success_handler)
    if failure_handler:
        submit_params.append("failure: " + failure_handler)
    if params:
        submit_params.append("params: " + json.JSONEncoder().encode(params))
    submit_params.append("submitEmptyText: false")
    return template % {
        'form_id': form.client_id,
        'invalid_handler': invalid_handler,
        'submit_params': ",".join(submit_params)
    }


def js_success_response():
    """
    :deprecated: Использовать OperationResult

    Возвращает Ext Ajax ответ что операция прошла успешно
    """
    return http.HttpResponse('{success: true}')


def js_close_window(win, forceClose=False):
    """
    :deprecated: Использовать методы окна, например,
    в ExtEditWindow метод calcelForm

    Возвращает JS код закрывающий окно win
    """
    assert isinstance(win, BaseExtWindow)
    forceClose = 'true' if forceClose else 'false'
    return 'function(){Ext.getCmp("%s").close(%s);}' % (
        win.client_id, forceClose)


def js_fire_event_window(event_name, close_after_fire=True, *args):
    """
    Генерирует вызов события для окна
    :param event_name: Название события
    :type event_name: str
    :param close_after_fire: Закрывать ли окно после вызова события?
    :type close_after_fire: bool
    :param *args: Параметры, которые будут переданы при генерации события
    """
    template = u'''
function(){
    win.fireEvent("%(event_name)s" %(params)s);
    %(win_close)s
}
    '''
    return template % {
        'event_name': event_name,
        'params': (',"%s"' % '","'.join(args)) if args else '',
        'win_close': 'win.close(true);' if close_after_fire else ''
    }


def js_on_key_enter(function, *args):
    return '''
function(field, e){
    if (e.getKey()== e.ENTER){
        return (%(function)s)(%(params)s);
    };
}
''' % {
        'function': function,
        'params': '"%s"' % '","'.join(args)
    }


class MessageBox(object):
    """
    Обёртка над стандартным MessageBox ExtJS
    """
    # Константы определяющие значек формы
    ICON_INFO = 'Ext.MessageBox.INFO'
    ICON_ERROR = 'Ext.MessageBox.ERROR'
    ICON_QUESTION = 'Ext.MessageBox.QUESTION'
    ICON_WARNING = 'Ext.MessageBox.WARNING'

    # Константы определяющие доступные кнопки
    BTN_OK = 'Ext.Msg.OK'
    BTN_CANCEL = 'Ext.Msg.CANCEL'
    BTN_OKCANCEL = 'Ext.Msg.OKCANCEL'
    BTN_YESNO = 'Ext.Msg.YESNO'
    BTN_YESNOCANCEL = 'Ext.Msg.YESNOCANCEL'

    def __init__(self, title='', msg='', icon=ICON_INFO, buttons=BTN_OK):
        self.title = title
        self.msg = msg
        self.icon = icon
        self.buttons = buttons
        # Обработчики для каждой кнопки
        self.handler_ok = ''
        self.handler_cancel = ''
        self.handler_yes = ''
        self.handler_no = ''

    def get_script(self):
        template = '''
Ext.Msg.show({
   title: "%(title)s",
   msg: "%(msg)s",
   buttons: %(buttons)s,
   fn: %(handler)s,
   animEl: 'elId',
   icon: %(icon)s
});
        '''
        template = template.replace('\n', ' ')
        # Для каждой кнопки можно задавать свой обработчик
        handler = '''
function(buttonId, text, opt){
    switch (buttonId){
        case 'yes':
            %(handler_yes)s
            break;
        case 'no':
            %(handler_no)s
            break;
        case 'cancel':
            %(handler_cancel)s
            break;
        case 'ok':
            %(handler_ok)s
            break;
    }
}
        '''
        handler = handler % {'handler_yes': self.handler_yes,
                             'handler_no': self.handler_no,
                             'handler_cancel': self.handler_cancel,
                             'handler_ok': self.handler_ok}
        result = template % {'title': self.title,
                             'msg': self.msg,
                             'buttons': self.buttons,
                             'icon': self.icon,
                             'handler': handler}
        return result




#coding:utf-8
"""
Created on 11.3.2010

@author: prefer
"""

from base import BaseExtPanel
from m3_ext.ui.base import ExtUIComponent, BaseExtComponent
from m3_ext.ui.containers import (
    ExtGridColumn, ExtGridBooleanColumn, ExtGridDateColumn, ExtGridNumberColumn
)


#==============================================================================
class ExtTree(BaseExtPanel):
    """
    Дерево с колонками
    """
    def __init__(self, *args, **kwargs):
        super(ExtTree, self).__init__(*args, **kwargs)
        self.template = 'ext-trees/ext-tree.js'

        # Типизированный список узлов
        self.nodes = []

        # Список колонок
        self._items = []

        # Специальный загрузчик для дерева
        self.tree_loader = ExtTreeLoader()

        # url для загрузки данных
        self.url = None

        # Текст для корневого элемента
        self.root_text = None

        # Если включен - показываем корневой элемент
        self.root_visible = False

        # Возможность использовать drag & drop.
        # То есть одновременные
        self.drag_drop = False

        # Если выставлен данный атрибут, то работает схема:
        # Всегда подгружается уровни дочерних элементов, в то время как
        # дочерние элементы уже подгружены.
        # Таким образом создается впечатление,
        # что дерево не динамическое и все узлы видны пользователю
        self.custom_load = False

        # Если включен - не рендерим drag'n'drop
        self.read_only = False

        # разрешить перетаскивать элементы в корень
        # (путем кидания просто в контейнер)
        self.allow_container_drop = True

        # Разрешает вставку узлов между родительскими элементами
        self.allow_parent_insert = False

        # Разрешить только дропить (перемещать в дерево)
        self.enable_drop = False

        # Разрешить только драгить (перемещать из дерево)
        self.enable_drag = False

        # перечень плагинов
        self.plugins = []

        self.init_component(*args, **kwargs)

    def _make_read_only(
            self, access_off=True, exclude_list=None, *args, **kwargs):
        exclude_list = exclude_list or []
        # Выключаем\включаем компоненты.
        super(ExtTree, self)._make_read_only(
            access_off, exclude_list, *args, **kwargs)

        self.read_only = access_off
        # контекстное меню.
        context_menu_items = [self.handler_contextmenu,
                              self.handler_containercontextmenu]
        for context_menu in context_menu_items:
            if (
                context_menu and
                hasattr(context_menu, 'items') and
                context_menu.items and
                hasattr(context_menu.items, '__iter__')
            ):
                for item in context_menu.items:
                    if isinstance(item, ExtUIComponent):
                        item.make_read_only(
                            self.read_only, exclude_list, *args, **kwargs)

    def t_render_tree_loader(self):
        return self.tree_loader.render()

    def t_render_nodes(self):
        return ','.join([node.render() for node in self.nodes])

    def t_render_root(self):
        return (
            "new Ext.tree.AsyncTreeNode({id: '-1', "
            "expanded: true, allowDrag: false, "
            "uiProvider: Ext.ux.tree.TreeGridNodeUI %s %s %s})"
        ) % (
            (',text:"%s"' % self.root_text) if self.root_text else '',
            (',%s:"%s"' % (self.columns[0].data_index, self.root_text)) if self.columns and self.root_text else '',
            (',children:[%s]' % ','.join(
                [node.render() for node in self.nodes])) if self.nodes else ''
        )

    def t_render_columns(self):
        return self.t_render_items()

    def add_nodes(self, *args):
        """
        Добавляет переданные узлы дерева
        :param *args: Узлы дерева
        """
        for node in args:
            self.nodes.append(node)

    def add_column(self, **kwargs):
        """
        Добавляет колонку с аргументами
        :param **kwargs: Аргументы
        """
        self.columns.append(ExtGridColumn(**kwargs))

    def add_bool_column(self, **kwargs):
        """
        Добавляет колонку с аргументами
        @param **kwargs: Аргументы
        """
        self.columns.append(ExtGridBooleanColumn(**kwargs))

    def add_number_column(self, **kwargs):
        """
        Добавляет колонку с аргументами
        :param **kwargs: Аргументы
        """
        self.columns.append(ExtGridNumberColumn(**kwargs))

    def add_date_column(self, **kwargs):
        """
        Добавляет колонку с аргументами
        :param **kwargs: Аргументы
        """
        self.columns.append(ExtGridDateColumn(**kwargs))

    @property
    def columns(self):
        return self._items

    @property
    def url(self):
        return self.__url

    @url.setter
    def url(self, value):
        self.tree_loader.url = value
        self.__url = value

    def pre_render(self):
        self.tree_loader.action_context = self.action_context
        super(ExtTree, self).pre_render()

    @property
    def handler_contextmenu(self):
        return self._listeners.get('contextmenu')

    @handler_contextmenu.setter
    def handler_contextmenu(self, menu):
        menu.container = self
        self._listeners['contextmenu'] = menu

    @property
    def handler_containercontextmenu(self):
        return self._listeners.get('containercontextmenu')

    @handler_containercontextmenu.setter
    def handler_containercontextmenu(self, menu):
        menu.container = self
        self._listeners['containercontextmenu'] = menu

    @property
    def handler_click(self):
        return self._listeners.get('click')

    @handler_click.setter
    def handler_click(self, function):
        self._listeners['click'] = function

    @property
    def handler_dblclick(self):
        return self._listeners.get('dblclick')

    @handler_dblclick.setter
    def handler_dblclick(self, function):
        self._listeners['dblclick'] = function

    @property
    def handler_dragdrop(self):
        return self._listeners.get('dragdrop')

    @handler_dragdrop.setter
    def handler_dragdrop(self, function):
        self._listeners['dragdrop'] = function

    @property
    def handler_dragover(self):
        return self._listeners.get('nodedragover')

    @handler_dragover.setter
    def handler_dragover(self, function):
        self._listeners['nodedragover'] = function

    @property
    def handler_startdrag(self):
        return self._listeners.get('startdrag')

    @handler_startdrag.setter
    def handler_startdrag(self, function):
        self._listeners['startdrag'] = function

    @property
    def handler_enddrag(self):
        return self._listeners.get('enddrag')

    @handler_enddrag.setter
    def handler_enddrag(self, function):
        self._listeners['enddrag'] = function

    @property
    def handler_drop(self):
        return self._listeners.get('nodedrop')

    @handler_drop.setter
    def handler_drop(self, function):
        self._listeners['nodedrop'] = function

    @property
    def handler_beforedrop(self):
        return self._listeners.get('beforenodedrop')

    @handler_beforedrop.setter
    def handler_beforedrop(self, function):
        self._listeners['beforenodedrop'] = function

    #--------------------------------------------------------------------------
    def render_base_config(self):
        super(ExtTree, self).render_base_config()
        for args in (
            ('useArrows', True),
            ('autoScroll', False),
            ('animate', True),
            ('containerScroll', True),
            ('columns', self.t_render_columns),
            ('loader', self.t_render_tree_loader),
            ('root', self.t_render_root),
            ('plugins', lambda: '[%s]' % ','.join(self.plugins)),
        ):
            self._put_config_value(*args)

        if self.drag_drop and not self.read_only:
            self._put_config_value('enableDD', True)
            self._put_config_value(
                'dropConfig',
                {
                    'allowContainerDrop': self.allow_container_drop,
                    'allowParentInsert': self.allow_parent_insert
                }
            )
        else:
            self._put_config_value('enableDrop', self.enable_drop)
            self._put_config_value('enableDrag', self.enable_drag)

    def render_params(self):
        super(ExtTree, self).render_params()
        self._put_params_value('customLoad', self.custom_load)
        self._put_params_value('rootVisible', self.root_visible)


#==============================================================================
class ExtTreeNode(ExtUIComponent):
    """Узел дерева"""

    def __init__(self, *args, **kwargs):
        super(ExtTreeNode, self).__init__(*args, **kwargs)
        self.template = 'ext-trees/ext-tree-node.js'

        # Отображаемый текст
        self.text = None

        # CSS класс для иконки
        self.icon_cls = None

        # True - Листьевой узел
        self.leaf = True

        # Имеются ли дочерние узлы
        # self.has_children = False

        # Развернут ли элемент
        self.expanded = False

        #
        self.auto_check = False

        # Отмечен ли галкой данный узел
        self.checked = False

        # Может ли узел быть отмеченым
        self.can_check = False

        # Типизированный список дочерних узлов
        self.children = []

        self.__items = {}
        self.init_component(*args, **kwargs)

    @property
    def has_children(self):
        return not self.leaf

    @has_children.setter
    def has_children(self, val):
        self.leaf = not val

    def t_render_children(self):
        return '[%s]' % ','.join([child.render() for child in self.children])

    def add_children(self, children):
        """
        Добавляет дочерние узлы
        Если необходимо, здесь можно указать у узлов
        атрибут "parent" на текущий (родительский) узел
        """
        self.has_children = True
        self.children.append(children)

    # TODO: Сомнительный кусок --> В шаблоне используется items.items
    @property
    def items(self):
        return self.__items

    def set_items(self, **kwargs):
        for k, v in kwargs.items():
            self.items[k] = v
    # --<<


#==============================================================================
class ExtTreeLoader(BaseExtComponent):
    """
    Загрузчик данных для ExtTree
    """
    def __init__(self, *args, **kwargs):
        super(ExtTreeLoader, self).__init__(*args, **kwargs)
        self.template = 'ext-trees/ext-tree-loader.js'

        # url для данных
        self.url = None

        # Словарь с параметрами
        self._base_params = {}

        # Node builders
        self.ui_providers = {}

        self.init_component(*args, **kwargs)

    def _set_base_params(self, params):
        self._base_params.update(params)

    def _get_base_params(self):
        return self._base_params

    base_params = property(_get_base_params, _set_base_params)



#coding:utf-8

from simple import (ExtDateField, 
                    ExtStringField, 
                    ExtNumberField, 
                    ExtComboBox,
                    ExtTextArea,
                    ExtCheckBox,
                    ExtTimeField,
                    ExtHiddenField,
                    ExtDisplayField,
                    ExtRadio)
from complex import (ExtDictSelectField,
                     ExtMultiSelectField,
                     ExtSearchField,
                     ExtFileUploadField,
                     ExtImageUploadField)

from base import BaseExtField, BaseExtTriggerField


#coding:utf-8
"""
Хелперы, которые помогают формировать пользовательский интерфейс
"""

from django.db.models.query import QuerySet
from m3 import M3JSONEncoder


def paginated_json_data(query, start=0, limit=25):
    if isinstance(query, QuerySet):
        try:
            total = query.count()
        except AttributeError:
            total = 0
    else:
        total = len(query)
    if start > 0 and limit < 1:
        data = list(query[start:])
    elif start >= 0 and limit > 0:
        data = list(query[start: start + limit])
    else:
        data = list(query)
    return M3JSONEncoder().encode({'rows': data, 'total': total})


def grid_json_data(query):
    """
    Выдает данные, упакованные в формате, пригодном для хаванья стором грида
    """
    return M3JSONEncoder().encode({'rows': list(query)})



#coding:utf-8

from uuid import uuid4

from django.template import Context
from django.template.loader import get_template


def render_component(component):
    """
    :param component: компонент
    :type component: BaseExtComponent или наследник
    :rtype: str
    """
    context = Context({'component': component, 'self': component})
    template = get_template(component.template)
    return template.render(context)


def render_template(template_name, variables=None):
    """
    :param template_name: имя шаблона
    :type template_name: str
    :param variables: словарь параметров
    :type variables: dict
    :rtype: str
    """
    context = Context(variables or {})
    template = get_template(template_name)
    return template.render(context)


def normalize(str):
    """
    Конвертирует строку в вид, понятный javascript'у
    :param str: строка
    :type str: str
    :rtype: str
    """
    return str.replace(
        '\r', '\\r'
    ).replace(
        '\n', '\\n'
    ).replace(
        '"', '\\"'
    ).replace(
        "'", "\\'"
    )


def generate_client_id():
    """
    Генерирует уникальный id для визуального компонента.
    :rtype: str
    """
    return 'cmp_' + str(uuid4())[0:8]


def get_img_size(src_size, dest_size):
    """
    Возвращает размеры изображения в пропорции с оригиналом исходя из того,
    как направлено изображение (вертикально или горизонтально)
    :param src_size: размер оригинала
    :type src_size: list / tuple
    :param dest_size: конечные размеры
    :type dest_size: list / tuple
    :rtype: tuple
    """
    width, height = dest_size
    src_width, src_height = src_size
    if height >= width:
        return (int(float(width) / height * src_height), src_height)
    return (src_width, int(float(height) / width * src_width))


def generate_id():
    """
    Генерирует восьмизначный random.
    :rtype: str
    """
    return str(uuid4())[0:8]



# coding:utf-8
"""
Базовые окна редактирования

Created on 14.12.2010

@author: akvarats
"""

from m3_ext.ui import windows
from m3_ext.ui import panels
from m3_ext.ui import controls
from m3_ext.ui import containers
from m3.actions import Action, ControllerCache


class GearEditWindow(windows.ExtEditWindow):
    """
    Окно редактирования, в котором лежит форма
    и в котором есть кнопки OK и Отмена.
    Размеры окна по умолчанию 600x400
    """
    def __init__(self, *args, **kwargs):
        super(GearEditWindow, self).__init__(*args, **kwargs)

        self.frozen_size(600, 400)
        self.title = u'Не забудь написать заголовок'

        self.form = panels.ExtForm()

        self.btn_save = controls.ExtButton(
            text=u'Сохранить',
            handler='submitForm'
        )
        self.btn_cancel = controls.ExtButton(
            name='cancel_btn',
            text=u'Отмена',
            handler='cancelForm'
        )

        self.buttons.extend([self.btn_save, self.btn_cancel])

        self._submit_action = None

    def frozen_size(self, width, height):
        """
        Устанавливает размер и заодно делает его минимально допустимым
        """
        self.width, self.height = width, height
        self.min_width, self.min_height = width, height

    def _set_submit_action(self, value):
        """ Из переданного типа экшена пытается получить адрес для формы """
        if isinstance(value, str):
            self.form.url = ControllerCache.get_action_url(value)
        elif isinstance(value, Action):
            self.form.url = value.get_absolute_url()
        elif issubclass(value, Action):
            self.form.url = value.absolute_url()

    submit_action = property(
        lambda self: self._submit_action,
        _set_submit_action
    )


class GearTableEditWindow(GearEditWindow):
    """
    Окно редактирования с лежащим внутри табличным контейнером.

    Количество строк и столбцов необходимо
    передавать через параметры конструкта:
    * columns: количество столбцов (по умолчаию - 2)
    * rows: количество строк (по умолчанию - 4)
    """
    def __init__(self, columns=2, rows=4, *args, **kwargs):
        super(GearTableEditWindow, self).__init__(*args, **kwargs)

        self.table = containers.ExtContainerTable(columns=columns, rows=rows)
        self.form.items.append(self.table)



# coding: utf-8
"""
Модуль с преднастроенными панелями-гридами

Created on 26.05.2010

@author: akvarats
"""
import json

from m3_ext.ui import containers, controls, menus, misc, render_component
from m3_ext.ui.base import ExtUIComponent, BaseExtComponent
from m3.actions.urls import get_url
from m3_ext.ui.containers.grids import ExtGridCheckBoxSelModel


class ExtObjectGrid(containers.ExtGrid):
    """
    Панель с гридом для управления списком объектов.
    """
    #==========================================================================
    # Внутренние классы для ExtObjectGrid
    #=========================================================================
    class GridContextMenu(menus.ExtContextMenu):
        """
        Внутренний класс для удобной работы с контекстным меню грида
        """
        def __init__(self, *args, **kwargs):
            super(ExtObjectGrid.GridContextMenu, self).__init__(
                *args, **kwargs)
            self.menuitem_new = menus.ExtContextMenuItem(
                text=u'Добавить',
                icon_cls='add_item',
                handler='contextMenuNew'
            )
            self.menuitem_edit = menus.ExtContextMenuItem(
                text=u'Изменить',
                icon_cls='edit_item',
                handler='contextMenuEdit'
            )
            self.menuitem_delete = menus.ExtContextMenuItem(
                text=u'Удалить',
                icon_cls='delete_item',
                handler='contextMenuDelete'
            )
            self.menuitem_separator = menus.ExtContextMenuSeparator()

            self.init_component()

    class GridTopBar(containers.ExtToolBar):
        """
        Внутренний класс для удобной работы топбаром грида
        """
        def __init__(self, *args, **kwargs):
            super(ExtObjectGrid.GridTopBar, self).__init__(*args, **kwargs)
            self.button_new = controls.ExtButton(
                text=u'Добавить',
                icon_cls='add_item',
                handler='topBarNew'
            )
            self.button_edit = controls.ExtButton(
                text=u'Изменить',
                icon_cls='edit_item',
                handler='topBarEdit'
            )
            self.button_delete = controls.ExtButton(
                text=u'Удалить',
                icon_cls='delete_item',
                handler='topBarDelete'
            )
            self.button_refresh = controls.ExtButton(
                text=u'Обновить',
                icon_cls='x-tbar-loading',
                handler='topBarRefresh'
            )

            self.items.extend([
                self.button_new,
                self.button_edit,
                self.button_delete,
                self.button_refresh,
            ])

            self.init_component()

    #==========================================================================
    # Собственно определение класса ExtObjectGrid
    #=========================================================================

    def __init__(self, *args, **kwargs):
        super(ExtObjectGrid, self).__init__(*args, **kwargs)
        self.template = 'ext-grids/ext-object-grid.js'

        #======================================================================
        # Действия, выполняемые изнутри грида
        #======================================================================

        # Экшен для новой записи
        self.action_new = None

        # Экшен для  изменения
        self.action_edit = None

        # Экшен для удаления
        self.action_delete = None

        # Экшен для данных
        self.action_data = None

        # Адрес для новой записи. Адреса имеют приоритет над экшенами!
        self.url_new = None

        # Адрес для изменения
        self.url_edit = None

        # Адрес для удаления
        self.url_delete = None

        # Адрес для данных
        self.url_data = None

        # Флаг о состоянии грида.
        # True означает что грид предназначен только для чтения.
        self.read_only = False

        #======================================================================
        # Источник данных для грида
        #======================================================================

        # Стор для загрузки данных
        self.store = misc.ExtJsonStore(
            auto_load=True, root='rows', id_property='id')

        # Признак того, маскировать ли грид при загрузки
        self.load_mask = True

        # Поля для id записи
        self.row_id_name = 'row_id'

        # имя параметра, через который передается имя выделенной колонки
        self.column_param_name = 'column'

        # Использовать постраничную навигацию
        self.allow_paging = True

        #======================================================================
        # Контекстное меню и бары грида
        #======================================================================

        # Контекстное меню для строки грида
        self.context_menu_row = ExtObjectGrid.GridContextMenu()

        # Контекстное меню для грида, если произошел счелчок не на строке
        self.context_menu_grid = ExtObjectGrid.GridContextMenu()

        # Топ бар для грида
        self.top_bar = ExtObjectGrid.GridTopBar()

        # Paging бар для постраничной навигации
        self.paging_bar = containers.ExtPagingBar()

        # Обработчик двойного клика
        self.dblclick_handler = 'onEditRecord'

        # Признак редактирования на клиенте
        # - особенным образом обрабатываются данные при редактировании
        self.local_edit = False

        # Атрибут store из store baseParams вынесен,
        # для одновременного изменения с атрибутом page_size paging_bar-а
        self._limit = self.store.limit if hasattr(self.store, 'limit') else -1

        self.init_component()

    def _make_read_only(
            self, access_off=True, exclude_list=(), *args, **kwargs):
        self.read_only = access_off
        # Выключаем\включаем компоненты.
        for item in (
            self.context_menu_grid.menuitem_new,
            self.context_menu_grid.menuitem_edit,
            self.context_menu_grid.menuitem_delete,
            self.context_menu_row.menuitem_new,
            self.context_menu_row.menuitem_edit,
            self.context_menu_row.menuitem_delete,
            self.context_menu_row,
        ):
            item.make_read_only(
                access_off, exclude_list, *args, **kwargs
            )
        if hasattr(self.top_bar, 'items') and self.top_bar.items:
            for item in self.top_bar.items:
                if hasattr(item, 'make_read_only') and callable(
                        item.make_read_only):
                    item.make_read_only(
                        access_off, exclude_list, *args, **kwargs)

    @property
    def handler_beforenew(self):
        return self._listeners.get('beforenewrequest')

    @handler_beforenew.setter
    def handler_beforenew(self, function):
        self._listeners['beforenewrequest'] = function

    @property
    def handler_beforeedit(self):
        return self._listeners.get('beforeeditrequest')

    @handler_beforeedit.setter
    def handler_beforeedit(self, function):
        self._listeners['beforeeditrequest'] = function

    def render(self):
        """
        Переопределяем рендер грида для того,
        чтобы модифицировать содержимое его
        панелей и контекстных меню
        """
        if self.action_new or self.url_new:
            self.context_menu_row.items.append(
                self.context_menu_row.menuitem_new)
            self.context_menu_grid.items.append(
                self.context_menu_grid.menuitem_new)

        if self.action_edit or self.url_edit:
            self.context_menu_row.items.append(
                self.context_menu_row.menuitem_edit)
            self.handler_dblclick = self.dblclick_handler

        if self.action_delete or self.url_delete:
            self.context_menu_row.items.append(
                self.context_menu_row.menuitem_delete)

        # контекстное меню прицепляется к гриду только в том случае, если
        # в нем есть хотя бы один пункт
        if self.context_menu_grid.items:
            self.handler_contextmenu = self.context_menu_grid
        if self.context_menu_row.items:
            self.handler_rowcontextmenu = self.context_menu_row

        #======================================================================
        # Настройка top bar
        #======================================================================
        def remove(this):
            if this in self.top_bar.items:
                self.top_bar.items.remove(this)

        # @TODO: Отрефакторить данный метод, чтобы он был не в рендеринге
        if (not self.action_data and not self.url_data and
                self.top_bar.button_refresh in self.top_bar.items):
            remove(self.top_bar.button_refresh)

        if (not self.action_delete and not self.url_delete and
                self.top_bar.button_delete in self.top_bar.items):
            remove(self.top_bar.button_delete)

        if (not self.action_edit and not self.url_edit and
                self.top_bar.button_edit in self.top_bar.items):
            remove(self.top_bar.button_edit)

        if (not self.action_new and not self.url_new and
                self.top_bar.button_new in self.top_bar.items):
            remove(self.top_bar.button_new)

        # тонкая настройка self.store
        if not self.store.url and self.action_data:
            self.store.url = get_url(self.action_data)

        if self.url_data:
            self.store.url = self.url_data

        # Стор может быть пустой
        # assert self.store.url, 'Url for store or action_data is not define'

        if self.allow_paging:
            # Значение self.store.start и так будет равно 0

            # Если store не экземпляр ExtJsonStore,
            # то у него нет атрибута limit
            if hasattr(self.store, 'limit'):
                self.store.limit = (
                    self.store.limit if self.store.limit > 0 else 25)
            self.bottom_bar = self.paging_bar

        self.render_base_config()
        self.render_params()
        return render_component(self)

    def render_params(self):
        super(ExtObjectGrid, self).render_params()

        # Получение адресов для грида.
        # Текстовые адреса более приоритетны чем экшены!
        if not self.url_new and self.action_new:
            self.url_new = get_url(self.action_new)
        if not self.url_edit and self.action_edit:
            self.url_edit = get_url(self.action_edit)
        if not self.url_delete and self.action_delete:
            self.url_delete = get_url(self.action_delete)
        if not self.url_data and self.action_data:
            self.url_data = get_url(self.action_data)

        context_json = (
            self.action_context.json if self.action_context else None)

        self._put_params_value(
            'actions',
            {
                'newUrl': self.url_new,
                'editUrl': self.url_edit,
                'deleteUrl': self.url_delete,
                'dataUrl': self.url_data,
                'contextJson': context_json})

        self._put_params_value('rowIdName', self.row_id_name)
        self._put_params_value('columnParamName', self.column_param_name)
        self._put_params_value('allowPaging', self.allow_paging)
        self._put_params_value('readOnly', self.read_only)
        self._put_params_value('localEdit', self.local_edit)

    def t_render_base_config(self):
        return self._get_config_str()

    def t_render_params(self):
        return self._get_params_str()

    @property
    def limit(self):
        return self._limit

    @limit.setter
    def limit(self, limit):

        self._limit = limit
        # Если не экземпляр ExtJSONStore, то у него нет атрибута limit.
        if hasattr(self.store, 'limit'):
            self.store.limit = limit

        # Размер страниц устанавливаем только,
        # если позволена постраничная навигация.
        if self.allow_paging:
            self.paging_bar.page_size = limit


class ExtMultiGroupinGrid(containers.ExtGrid):
    """
    Грид с возможностью множественной группировки колонок.
    Обработка группировки происходит на сервере
    .. seealso::
        m3.helpers.datagrouping
    """

    class GridExportMenu(menus.ExtContextMenu):
        """
        Внутренний класс для удобной работы с контекстным меню грида
        """

        def __init__(self, *args, **kwargs):
            super(ExtMultiGroupinGrid.GridExportMenu, self).__init__(
                *args, **kwargs)
            self.xls = menus.ExtContextMenuItem(
                text=u'XLS (Excel2003 до 65000 строк)',
                handler='function(){exportData("xls");}')
            self.csv = menus.ExtContextMenuItem(
                text=u'CSV (разделитель ";")',
                handler='function(){exportData("csv");}')

            self.items.extend([
                self.xls,
                self.csv,
            ])

            self.init_component()

    class LiveGridTopBar(containers.ExtToolBar):
        """
        Внутренний класс для удобной работы топбаром грида
        """
        def __init__(self, *args, **kwargs):
            super(ExtMultiGroupinGrid.LiveGridTopBar, self).__init__(
                *args, **kwargs)
            self._ext_name = "Ext.ux.grid.livegrid.Toolbar"
            self.button_new = controls.ExtButton(
                text=u'Добавить',
                icon_cls='add_item',
                handler='topBarNew'
            )
            self.button_edit = controls.ExtButton(
                text=u'Изменить',
                icon_cls='edit_item',
                handler='topBarEdit'
            )
            self.button_delete = controls.ExtButton(
                text=u'Удалить',
                icon_cls='delete_item',
                handler='topBarDelete'
            )
            self.button_export = controls.ExtButton(
                text=u'Экспорт',
                icon_cls='icon-table-go',
                menu=ExtMultiGroupinGrid.GridExportMenu(
                )
            )

            self.items.extend([
                self.button_new,
                self.button_edit,
                self.button_delete,
                self.button_export,
            ])

            self.init_component()

    # Поле в котором будет содержаться значение ключа группировки
    # должно отличаться от ключевого поля Store,
    # т.к. должно содержать совсем другие данные
    data_id_field = 'id'

    # Поле отображаемое вместо идентификатора группировки
    # (по-умолчанию отображается сам идентификатор)
    data_display_field = 'id'

    # Поля группировки по-умолчанию (список имен полей)
    grouped = None

    def __init__(self, *args, **kwargs):
        super(ExtMultiGroupinGrid, self).__init__(*args, **kwargs)
        self.template = 'ext-grids/ext-multigrouping-grid.js'
        # Для данных
        self.action_data = None

        self.action_new = None  # Экшен для новой записи
        self.action_edit = None  # Экшен для  изменения
        self.action_delete = None  # Экшен для удаления
        self.action_export = None  # Экшен для экспорта

        # Поля для id записи
        self.row_id_name = 'row_id'

        # Обработчик двойного клика
        self.dblclick_handler = 'onEditRecord'

        # Топ бар для грида
        self._top_bar = ExtMultiGroupinGrid.LiveGridTopBar()

        # Признак того, маскировать ли грид при загрузки
        self.load_mask = True

        # Стор для загрузки данных
        self.store = misc.store.ExtMultiGroupingStore(
            auto_load=True, root='rows', id_property='index')

        # Начальный перечень сгруппированных колонок
        self.grouped = []

        # Признак редактирования на клиенте
        # - особенным образом обрабатываются данные при редактировании
        self.local_edit = False

        # Признак отображения всплывающих подсказок
        self.show_tooltips = True

        # Признак возможности группировки (показывает панель)
        self.groupable = True

        # Признак отображения информации о записях
        self.display_info = True

        # Формат отображения информации о записях
        self.display_message = u'Показано {0}-{1} из {2}'

        # Объем буфера записей - должен быть больше,
        # чем число соседних элементов + число видимых строк
        self.buffer_size = 200

        # Число соседних элементов сверху и снизу от видимой области,
        # для предотвращения запросов.
        # Обычно 1/4 или 1/2 от объема буфера
        self.near_limit = 100

        # Стиль заголовка колонки.
        # Применяется ко всем колонкам. Например: 'text-align: center;'
        self.header_style = ''

        # Url для пака
        self.url_data = self.url_new = self.url_edit = self.url_delete = None

        self.init_component()

    def render(self):
        self.store.url = self.store.url or self.url_data

        assert self.store.url or self.action_data, 'Action data is not defined'
        # тонкая настройка self.store
        if not self.store.url and self.action_data:
            self.store.url = get_url(self.action_data)

        self.render_base_config()
        self.render_params()
        return render_component(self)

    def render_base_config(self):
        # Добавление происходит перед вызовом родителя,
        # т.к. в нём _view_config уже будет сериализован
        if self.header_style:
            self._view_config['headerStyle'] = self.header_style
        super(ExtMultiGroupinGrid, self).render_base_config()

    def render_params(self):
        super(ExtMultiGroupinGrid, self).render_params()

        data_url = self.url_data or (get_url(
            self.action_data) if self.action_data else None)

        def remove(this):
            if this in self._top_bar.items:
                self._top_bar.items.remove(this)

        new_url = self.url_new or (get_url(
            self.action_new) if self.action_new else None)
        if not new_url:
            remove(self._top_bar.button_new)

        edit_url = self.url_edit or (get_url(
            self.action_edit) if self.action_edit else None)
        if not edit_url:
            remove(self._top_bar.button_edit)
        else:
            self.handler_dblclick = self.dblclick_handler

        delete_url = self.url_delete or (get_url(
            self.action_delete) if self.action_delete else None)
        if not delete_url:
            remove(self._top_bar.button_delete)

        export_url = get_url(
            self.action_export) if self.action_export else None
        if not self.action_export:
            remove(self._top_bar.button_export)
        context_json = (
            self.action_context.json if self.action_context else None)

        for args in (
            (
                'actions',
                {
                    'dataUrl': data_url,
                    'newUrl': new_url,
                    'editUrl': edit_url,
                    'deleteUrl': delete_url,
                    'exportUrl': export_url,
                    'contextJson': context_json
                }
            ),
            (
                'groupedColumns',
                lambda: '[%s]' % ','.join(
                    ["'%s'" % col for col in self.grouped]
                )
            ),
            ('dataIdField', self.data_id_field),
            ('dataDisplayField', self.data_display_field),
            ('toolbar', self._top_bar.t_render_items),
            ('rowIdName', self.row_id_name),
            ('localEdit', self.local_edit),
            ('groupable', self.groupable),
            ('showTooltips', self.show_tooltips),
            ('displayInfo', self.display_info),
            ('displayMsg', self.display_message),
            ('bufferSize', self.buffer_size),
            ('nearLimit', self.near_limit),
        ):
            self._put_params_value(*args)

    def t_render_base_config(self):
        return self._get_config_str()

    def t_render_params(self):
        return self._get_params_str()

    def t_render_plugins(self):
        if self.show_tooltips:
            tooltips = []
            for column in self.columns:
                if column.tooltip:
                    tooltips.append(column.tooltip)
                elif column.data_index:
                    tooltips.append({
                        'field': column.data_index,
                        'tpl': '{%s}' % column.data_index
                    })
            self.plugins.append(
                'new Ext.ux.plugins.grid.CellToolTips(%s)' % json.dumps(tooltips))
        return super(ExtMultiGroupinGrid, self).t_render_plugins()

    @property
    def handler_beforenew(self):
        return self._listeners.get('beforenewrequest')

    @handler_beforenew.setter
    def handler_beforenew(self, function):
        self._listeners['beforenewrequest'] = function

    @property
    def handler_beforeedit(self):
        return self._listeners.get('beforeeditrequest')

    @handler_beforeedit.setter
    def handler_beforeedit(self, function):
        self._listeners['beforeeditrequest'] = function

    def _make_read_only(
            self, access_off=True, exclude_list=(), *args, **kwargs):

        super(ExtMultiGroupinGrid, self)._make_read_only(
            access_off, exclude_list, *args, **kwargs)

        if (self._top_bar and hasattr(self._top_bar, 'items')
            and self._top_bar.items and hasattr(
                self._top_bar.items, '__iter__')):
            for item in self._top_bar.items:
                if isinstance(item, ExtUIComponent):
                    item.make_read_only(
                        self.read_only,
                        exclude_list,
                        *args, **kwargs
                    )


class ExtObjectSelectionPanel(containers.ExtContainer):
    """
    Класс, совмещающий возможность ObjectGrid'a
    и возможности выбора и запоминания значений в случае Paging'a
    """

    def __init__(self,
                 grid=None,
                 selection_columns=None,
                 selection_grid_conf=None,
                 *args,
                 **kwargs):
        super(ExtObjectSelectionPanel, self).__init__(*args, **kwargs)

        #self.xtype = 'object-selection-panel'
        self.layout = 'border'
        self.grid = grid
        self.selection_grid_conf = selection_grid_conf or {}
        self.selection_columns = selection_columns or []

        self._ext_name = 'Ext.m3.ObjectSelectionPanel'

    def render_base_config(self):
        super(ExtObjectSelectionPanel, self).render_base_config()
        assert self.grid, 'Grid should be define'

        if not isinstance(self.grid.sm, ExtGridCheckBoxSelModel):
            self.grid.sm = ExtGridCheckBoxSelModel()

        self._put_config_value('grid', self.grid.render)
        self._put_config_value('selectionColumns', lambda: json.dumps(self.selection_columns))
        self._put_config_value('selectionGridConf', self._render_selection_conf)

    def _render_selection_conf(self):
        """
        Рендеринг параметров грида для выделения записей
        :return: str
        """
        res = []
        for k, v in self.selection_grid_conf.items():
            if isinstance(v, BaseExtComponent):
                res.append('%s:%s' % (k, v.render()))
            else:
                res.append('%s:%s' % (k, json.dumps(v)))

        return '{%s}' % ','.join(res)



# -*- coding: utf-8 -*-
# Create your views here.rtn.core.views
from django.shortcuts import render_to_response
from django import template as django_template

from m3_ext.context_processors import DesktopProcessor


DEFAULT_PROCESSORS = (DesktopProcessor.process,)


def workspace(
    template='m3_workspace.html',
    context_processors=DEFAULT_PROCESSORS
):
    u"""
    Возвращает view для тображения Рабочего Стола
    на основе указанного шаблона
    :param template: имя файла шаблона
    :type template: str
    """
    from django.conf import settings
    def workspace_view(request):
        u"""
        view для отображения Рабочего Стола
        """
        context = django_template.RequestContext(
            request, {'DEBUG': settings.DEBUG},
            processors=context_processors)
        return render_to_response(template, context)
    return workspace_view



#coding:utf-8
"""
Created on 27.02.2010

@author: akvarats
@author: prefer <telepenin@bars-open.ru>
"""
from m3_ext.ui.base import ExtUIComponent
from m3_ext.ui.controls import ExtButton

from base import BaseExtContainer


#==============================================================================
class ExtContainer(BaseExtContainer):
    """
    Класс, имитирующий работу Ext.Container
    """
    def __init__(self, *args, **kwargs):
        super(ExtContainer, self).__init__(*args, **kwargs)
        self._ext_name = 'Ext.Container'
        self.init_component(*args, **kwargs)

    @property
    def items(self):
        return self._items

    def render_base_config(self):
        super(ExtContainer, self).render_base_config()
        if self._items:
            self._put_config_value('items', self.t_render_items)

    def render(self):
        if 'overflow' not in self.style:
            # делается для хака IE иначе иногда дочерние элементы ведут себя
            # словно у них задано position: fixed т.е. начинаю неадекватничать
            self.style['overflow'] = 'hidden'
        assert getattr(self, '_ext_name'), (
            'Class %s is not define "_ext_name"' %
            self.__class__.__name__
        )

        self.pre_render()

        try:
            self.render_base_config()
            self.render_params()
        except UnicodeDecodeError:
            raise Exception('Some attribute is not unicode')

        base_config = self._get_config_str()
        params = self._get_params_str()
        res = '%(ext_name)s({%(base_config)s},{%(params)s})' % {
            'ext_name': self._ext_name,
            'base_config': base_config,
            'params': params
        }

        return 'new %s' % res if not self._is_function_render else res


class ExtToolBar(BaseExtContainer):
    """
    Класс, имитирующий работу Ext.Toolbar
    """

    class Fill(object):
        def render(self):
            return '"->"'

    class Separator(object):
        def render(self):
            return '"-"'

    class Spacer(object):
        def __init__(self, width=2):
            self.width = width

        def render(self):
            return "{xtype: 'tbspacer', width: %d}" % self.width

    class TextItem(object):
        def __init__(self, text='', handler=None):
            self.text = text
            self.handler = handler

        def render(self):
            if self.handler:
                return '{text:"%s", handler: %s}' % (self.text, self.handler)
            else:
                return '"%s"' % self.text

    def __init__(self, *args, **kwargs):
        super(ExtToolBar, self).__init__(*args, **kwargs)
        self._ext_name = 'Ext.Toolbar'
        self._items = []
        self.init_component(*args, **kwargs)

    def t_render_items(self):
        res = []
        for item in self.items:
            # Если объект структуры классов, то пусть сам рендерится,
            # если нет - так как есть.
            if hasattr(item, 'render') and callable(item.render):
                res.append(item.render())
            else:
                res.append(item)
        return '[%s]' % ','.join(res)

    def add_fill(self):
        """
        Переносит все последующие компоненты направо
        """
        self.items.append(ExtToolBar.Fill())

    def add_separator(self):
        """
        Добавляет разделитель
        """
        self.items.append(ExtToolBar.Separator())

    def add_spacer(self, width=2):
        """
        Добавляет дополнительное расстояние с шириной width
        :param width: расстояние
        :type width: int
        """
        self.items.append(ExtToolBar.Spacer(width))

    def add_text_item(self, text_item):
        """
        Добавляет текст
        :param text_item: текст
        :type text_item: unicode
        """
        self.items.append(ExtToolBar.TextItem(text_item))

    def add_menu(self, **kwargs):
        """
        Добавляет меню
        :param **kwargs: конфиг для меню
        """
        self.items.append(ExtToolbarMenu(**kwargs))

    @property
    def items(self):
        """
        Список вложенных компонентов
        """
        return self._items

    def render_base_config(self):
        super(ExtToolBar, self).render_base_config()
        if self.items:
            self._put_config_value('items', self.t_render_items)

    def render(self):
        assert getattr(self, '_ext_name'), (
            'Class %s is not define "_ext_name"' %
            self.__class__.__name__
        )

        self.pre_render()

        try:
            self.render_base_config()
            self.render_params()
        except UnicodeDecodeError:
            raise Exception('Some attribute is not unicode')
        except Exception as msg:
            raise Exception(msg)

        base_config = self._get_config_str()
        params = self._get_params_str()
        res = '%(ext_name)s({%(base_config)s},{%(params)s})' % {
            'ext_name': self._ext_name,
            'base_config': base_config,
            'params': params
        }

        return 'new %s' % res if not self._is_function_render else res


class ExtStaticToolBarItem(ExtUIComponent):
    """
    :deprecated: Нужно использовать встроенные подклассы в качестве элементов

    Преднастроенные элементы в тулбаре
    @TODO: Для чего нужнен отдельный класс, если задача может решится методами
    тулбара?
    """
    def __init__(self, static_value='', *args, **kwargs):
        super(ExtStaticToolBarItem, self).__init__(*args, **kwargs)
        self.static_value = static_value
        self.init_component(*args, **kwargs)

    def render(self):
        return self.static_value

    def _make_read_only(
            self, access_off=True, exclude_list=(), *args, **kwargs):
        # Описание в базовом классе ExtUiComponent.
        pass


class ExtTextToolBarItem(ExtUIComponent):
    """
    :deprecated: Нужно использовать встроенные подклассы в качестве элементов

    Текстовый элемент в тулбаре
    @TODO: Для чего нужнен отдельный класс, если задача может решится методами
    тулбара?
    """
    def __init__(self, static_value='', *args, **kwargs):
        super(ExtTextToolBarItem, self).__init__(*args, **kwargs)
        self.text = None
        self.init_component(*args, **kwargs)

    def render(self):
        return "{xtype: 'tbtext', text: '%s'}" % self.text

    def _make_read_only(
            self, access_off=True, exclude_list=(), *args, **kwargs):
        # Описание в базовом классе ExtUiComponent.
        pass


class ExtPagingBar(BaseExtContainer):
    """
    Класс, имитирующий работу Ext.PagingToolbar
    """
    def __init__(self, *args, **kwargs):
        super(ExtPagingBar, self).__init__(*args, **kwargs)
        self._ext_name = 'Ext.PagingToolbar'

        self.page_size = 25
        self.display_message = u'Показано записей {0} - {1} из {2}'
        self.display_info = True
        self.empty_message = u'Нет записей'
        self.init_component(*args, **kwargs)

    def render_base_config(self):
        super(ExtPagingBar, self).render_base_config()
        self._put_config_value('pageSize', self.page_size)
        self._put_config_value('displayInfo', self.display_info)
        self._put_config_value('displayMsg', self.display_message)
        self._put_config_value('emptyMsg', self.empty_message)

    def render(self):
        assert getattr(self, '_ext_name'), (
            'Class %s is not define "_ext_name"' %
            self.__class__.__name__
        )

        self.pre_render()

        try:
            self.render_base_config()
            self.render_params()
        except UnicodeDecodeError:
            raise Exception('Some attribute is not unicode')
        except Exception as msg:
            raise Exception(msg)

        base_config = self._get_config_str()
        params = self._get_params_str()
        res = '%(ext_name)s({%(base_config)s},{%(params)s})' % {
            'ext_name': self._ext_name,
            'base_config': base_config,
            'params': params
        }

        return 'new %s' % res if not self._is_function_render else res


class ExtToolbarMenu(ExtUIComponent):
    """
    Класс, позволяющий легко вставлять меню в ToolBar
    """
    def __init__(self, *args, **kwargs):
        super(ExtToolbarMenu, self).__init__(*args, **kwargs)
        self.text = None
        self.icon_cls = None
        self.tooltip_text = None
        self.menu = None
        self.init_component(*args, **kwargs)

    def render(self):
        res = 'id:"%s"' % self.client_id
        if self.text:
            res = 'text: "%s"' % self.text
        if self.icon_cls:
            res += ',iconCls: "%s"' % self.icon_cls
        if self.tooltip_text:
            res += ',tooltip: "%s"' % self.tooltip_text
        if self.menu:
            res += ',menu: %s' % self.menu.render()

        return '{%s}' % res

    def _make_read_only(
            self, access_off=True, exclude_list=(), *args, **kwargs):
        if self.menu:
            self.menu.make_read_only(
                access_off, exclude_list, *args, **kwargs)


class ExtRadioGroup(BaseExtContainer):
    """
    Компонент-контейнер для радио-полей
    """

    def __init__(self, *args, **kwargs):
        super(ExtRadioGroup, self).__init__(*args, **kwargs)
        self._ext_name = 'Ext.form.RadioGroup'

        # количество колонок в контейнере
        self.columns = None
        self.label = None

        self.init_component(*args, **kwargs)

    @property
    def items(self):
        return self._items

    def render_base_config(self):
        super(ExtRadioGroup, self).render_base_config()
        if self.items:
            self._put_config_value('items', self.t_render_items)
        if self.columns:
            self._put_config_value('columns', self.columns)

    def render(self):
        try:
            self.render_base_config()
        except Exception as msg:
            raise Exception(msg)

        base_config = self._get_config_str()
        return 'new %s({%s})' % (self._ext_name, base_config)



#coding:utf-8
"""
Created on 3.3.2010

@author: prefer
"""
import json
import abc
import datetime

from decimal import Decimal

from m3_ext.ui import normalize

from m3_ext.ui.base import BaseExtComponent

from base_store import BaseExtStore


class ExtDataStore(BaseExtStore):
    """
    Хранилище данных, которое не генерирует запрос на сервер,
    а принимает данные в массиве data, либо через метод load_data
    """
    def __init__(self, data=None, *args, **kwargs):
        super(ExtDataStore, self).__init__(*args, **kwargs)

        # По умолчанию первым параметром передаются данные на заполнение store
        if data:
            self.data = data
        else:
            self.data = []

        self.reader = ExtArrayReader()

        self._id_property = 'id'

        self._root = None

        self._total_property = None

        # Для заполнения полей в шаблоне
        self.__columns = []

        self.init_component(*args, **kwargs)

    @property
    def id_property(self):
        return self.reader.id_property

    @id_property.setter
    def id_property(self, value):
        self.reader.id_property = value

    @property
    def root(self):
        return self.reader.root

    @root.setter
    def root(self, value):
        self.reader.root = value

    @property
    def total_property(self):
        return self.reader.total_property

    @total_property.setter
    def total_property(self, value):
        self.reader.total_property = value

    def render_base_config(self):

        super(ExtDataStore, self).render_base_config()

        self._put_config_value('reader', self.reader.render)

        self._put_config_value('data', self.render_data)

    def load_data(self, data):
        self.data = data

    def render(self, columns):

        self.reader._fields = columns

        try:
            self.render_base_config()
        except UnicodeDecodeError as msg:
            raise Exception(msg)

        base_config = self._get_config_str()

        return 'new Ext.data.Store({%s})' % base_config

    def render_data(self):
        return self.reader._render_data(self.data)


class ExtJsonStore(BaseExtStore):
    """
    Хранилище данных, которое отправляет запрос на сервер и ждет, что данные
    вернуться в формате json
    """
    def __init__(self, *args, **kwargs):
        super(ExtJsonStore, self).__init__(*args, **kwargs)
        # TODO: Отрефакторить под внутриклассовый рендеринг
        self.template = 'ext-misc/ext-json-store.js'

        # Для заполнения полей в шаблоне
        self.__columns = []

        # Начальная позиция для показа,
        # если используется постраничная навигация
        self.__start = 0

        # Количество записей для показа,
        # если используется постраничная навигация
        self.__limit = -1

        #
        self.total_property = None

        # Название вершины в json массиве, откуда будут браться записи
        # Например root = 'rows'
        # Тогда предполагаемый json массив должен выглядеть примерно так:
        # {rows: [id:1, name:'name', age:45]}
        self.root = None

        # Использовать ли удаленную сортировку
        self.remote_sort = False

        # Поле, откуда будет браться id записи
        self.id_property = 'id'

        self.init_component(*args, **kwargs)

    def render(self, columns):
        self.__columns = columns
        #self.__columns.insert(0, self.id_property)
        return super(ExtJsonStore, self).render()

    def t_render_fields(self):
        """
        Прописывается в шаблоне и заполняется при рендеринге
        """

        res = ['{name: "%s"}' % self.id_property]
        for col in self.__columns:
            if isinstance(col, basestring):
                if col != self.id_property:
                    res.append('{name: "%s"}' % col)
            else:
                if col.data_index != self.id_property:
                    d = {'name': col.data_index}
                    if hasattr(col, 'format'):  # ExtDateField
                        d['type'] = 'date'
                        d['dateFormat'] = col.format

                    res.append(json.dumps(d))
        return ','.join(res)

    def _get_start(self):
        return self.__start

    def _set_start(self, s):
        self.__start = s
        self._base_params['start'] = self.__start

    start = property(_get_start, _set_start)

    def _get_limit(self):
        return self.__limit

    def _set_limit(self, l):
        self.__limit = l
        self._base_params['limit'] = self.__limit

    limit = property(_get_limit, _set_limit)


class ExtJsonWriter(BaseExtStore):
    """
    Предназначен для отправки и преобразования
    новых и измененных записей Store на сервер
    """
    def __init__(self, *args, **kwargs):
        super(ExtJsonWriter, self).__init__(*args, **kwargs)

        # Если True, записи (records) переводится в хешированные данные,
        # имя беруться из ридера (Reader).
        # .. seealso::
        # Подробности http://extjs.docs/d/?class=Ext.data.JsonWriter
        self.encode = True

        # Если False, при удалении будет отправляться
        # только id записи на сервер
        self.encode_delete = False

        # Если True, то сохраняются все записи, а не только измененные
        self.write_all_fields = False

        self.init_component(*args, **kwargs)

    def render(self):
        result = '''
new Ext.data.JsonWriter({
    %(writeAllFields)s
    %(encode)s
    %(encodeDelete)s
})
''' % {
            'writeAllFields': (
                'writeAllFields: true' if self.write_all_fields else ''),
            'encode': 'encode: false' if not self.encode else '',
            'encodeDelete': 'encodeDelete: true' if self.encode_delete else ''}

        return result


class ExtDataReader(BaseExtComponent):
    """
    Получает ответ от сервера и декодирует его в  массив Record-ов
    """

    __metaclass__ = abc.ABCMeta

    def __init__(self, *args, **kwargs):
        super(ExtDataReader, self).__init__(*args, **kwargs)

        # Поле, откуда будет браться идентификатор
        self.id_property = 'id'

        self.root = None

        self.total_property = None

        # Массив данных
        self._fields = []

        self.init_component(*args, **kwargs)

    def get_fields(self):
        return self._fields

    def set_fields(self, *args):

        for field in args:
            self._fields.append(field)

    def convert_value(self, value):
        """
        Конвертирует данные в JS строку
        """

        if value is None:
            res_value = '""'
        elif isinstance(value, bool):
            res_value = str(value).lower()
        elif isinstance(value, (int, Decimal, float)):
            res_value = str(value)
        elif isinstance(value, datetime.date):
            res_value = 'new Date("%s")' % value.ctime()
        else:
            res_value = '"%s"' % normalize(value)

        return res_value

    @abc.abstractmethod
    def _render_data(self, data):
        """
        """

    @abc.abstractmethod
    def _render_fields(self):
        """
        """

    def render(self):

        try:
            self.render_base_config()
        except UnicodeDecodeError as msg:
            raise Exception(msg)


class ExtJsonReader(ExtDataReader):
    """
    Reader для данных типа JSON
    """

    def render_base_config(self):

        self._put_config_value('idProperty', self.id_property)
        self._put_config_value('root', self.root)
        self._put_config_value('totalProperty', self.total_property)

    def _render_data(self, data):
        res = []
        for item in data:

            res_tmp = []
            for key, value in item.items():
                res_value = self.convert_value(value)
                res_tmp.append('%s: %s' % (key, res_value))

            res_tmp = ','.join(res_tmp)
            res_tmp = '{%s}' % res_tmp
            res.append(res_tmp)
        return '[%s]' % ','.join(res)

    def _render_fields(self):

        res = [
            '{name: "%s", mapping: "%s"}' %
            (self.id_property, self.id_property)
        ]
        for col in self.get_fields():
            if isinstance(col, basestring):
                if col != self.id_property:
                    res.append('{name: "%s", mapping: "%s"}' % (col, col))
            else:
                if col.data_index != self.id_property:
                    d = {'name': col.data_index}
                    if hasattr(col, 'format'):  # ExtDateField
                        d['type'] = 'date'
                        d['dateFormat'] = col.format
                    if hasattr(col, 'mapping'):
                        d['mapping'] = col.mapping
                    else:
                        d['mapping'] = col.data_index

                    res.append(json.dumps(d))
        return 'Ext.data.Record.create([%s])' % ','.join(res)

    def render(self):
        super(ExtJsonReader, self).render()

        return 'new Ext.data.JsonReader({%s}, %s)' % (
            self._get_config_str(), self._render_fields())


class ExtArrayReader(ExtDataReader):
    """
    Reader для данных в виде обычного массива
    """

    def __init__(self, *args, **kwargs):

        self.id_index = 0
        super(ExtArrayReader, self).__init__(*args, **kwargs)

    def _render_data(self, data):
        res = []
        for item in data:
            res_tmp = []
            for subitem in item:

                res_value = self.convert_value(subitem)

                res_tmp.append(res_value)

            res.append('[%s]' % ','.join(res_tmp))

        return '[%s]' % ','.join(res)

    def _render_fields(self):
        """
        Прописывается в шаблоне и заполняется при рендеринге
        """
        res = [
            '{name: "%s", mapping: %d}' %
            (self.id_property, 0)
        ]  # ID
        # .. note::
        # чтобы правильно выставить mapping надо определить,
        # есть ли в списке колонок поле с таким же именем
        # если такая колонка встречается, то пропускаем её
        ind = 1
        for i, col in enumerate(self.get_fields()):
            if isinstance(col, basestring):
                if col != self.id_property:
                    res.append('{name: "%s", mapping: %d}' % (col, ind+i))
                else:
                    ind = 0
            else:
                if col.data_index != self.id_property:
                    # 1-ое поле - ID
                    d = {'name': col.data_index, 'mapping': ind+i}
                    if hasattr(col, 'format'):  # ExtDateField
                        d['type'] = 'date'
                        d['dateFormat'] = col.format
                    res.append(json.dumps(d))
                else:
                    ind = 0
        return 'Ext.data.Record.create([%s])' % ','.join(res)

    def render_base_config(self):

        self._put_config_value('idIndex', self.id_index)

    def render(self):
        super(ExtArrayReader, self).render()

        return 'new Ext.data.ArrayReader({%s}, %s)' % (
            self._get_config_str(), self._render_fields())


class ExtGroupingStore(ExtJsonStore):
    """
    Хранилище используемое для группировки по определенным полям в гриде
    """
    def __init__(self, *args, **kwargs):
        super(ExtGroupingStore, self).__init__(*args, **kwargs)
        self.template = 'ext-misc/ext-grouping-store.js'

        self.reader = None

        # Серверная группировка
        self.remote_group = False

        # Имя поля, используемой для сортировки
        self.group_field = None

        # Объект, в котором может указываться например порядок сортировки
        # .. seealso::
        # http://extjs.docs/d/?class=Ext.data.GroupingStore
        self.sort_info = None
        self.init_component(*args, **kwargs)


class ExtMultiGroupingStore(ExtJsonStore):
    """
    Хранилище используемое для грида с множественной серверной группировкой
    """
    def __init__(self, *args, **kwargs):
        super(ExtMultiGroupingStore, self).__init__(*args, **kwargs)
        self.template = 'ext-misc/ext-livegrid-store.js'
        self.version_property = 'version'
        self.bufferSize = 200
        self.init_component(*args, **kwargs)



#coding:utf-8


class Icons:
    """
    Статический класс, который включает пак набора иконок и дефолтные иконки m3
    """
    # дефолтные иконки m3
    M3_ADD = 'add_item'
    M3_EDIT = 'edit_item'
    M3_DELETE = 'delete_item'
    M3_SEARCH = 'search'

    # Сгенерированный набор иконок
    ACCEPT = 'icon-accept'
    ADD = 'icon-add'
    ANCHOR = 'icon-anchor'
    APPLICATION = 'icon-application'
    APPLICATION_ADD = 'icon-application-add'
    APPLICATION_CASCADE = 'icon-application-cascade'
    APPLICATION_DELETE = 'icon-application-delete'
    APPLICATION_DOUBLE = 'icon-application-double'
    APPLICATION_EDIT = 'icon-application-edit'
    APPLICATION_ERROR = 'icon-application-error'
    APPLICATION_FORM = 'icon-application-form'
    APPLICATION_FORM_ADD = 'icon-application-form-add'
    APPLICATION_FORM_DELETE = 'icon-application-form-delete'
    APPLICATION_FORM_EDIT = 'icon-application-form-edit'
    APPLICATION_FORM_MAGNIFY = 'icon-application-form-magnify'
    APPLICATION_GET = 'icon-application-get'
    APPLICATION_GO = 'icon-application-go'
    APPLICATION_HOME = 'icon-application-home'
    APPLICATION_KEY = 'icon-application-key'
    APPLICATION_LIGHTNING = 'icon-application-lightning'
    APPLICATION_LINK = 'icon-application-link'
    APPLICATION_OSX = 'icon-application-osx'
    APPLICATION_OSX_TERMINAL = 'icon-application-osx-terminal'
    APPLICATION_PUT = 'icon-application-put'
    APPLICATION_SIDE_BOXES = 'icon-application-side-boxes'
    APPLICATION_SIDE_CONTRACT = 'icon-application-side-contract'
    APPLICATION_SIDE_EXPAND = 'icon-application-side-expand'
    APPLICATION_SIDE_LIST = 'icon-application-side-list'
    APPLICATION_SIDE_TREE = 'icon-application-side-tree'
    APPLICATION_SPLIT = 'icon-application-split'
    APPLICATION_TILE_HORIZONTAL = 'icon-application-tile-horizontal'
    APPLICATION_TILE_VERTICAL = 'icon-application-tile-vertical'
    APPLICATION_VIEW_COLUMNS = 'icon-application-view-columns'
    APPLICATION_VIEW_DETAIL = 'icon-application-view-detail'
    APPLICATION_VIEW_GALLERY = 'icon-application-view-gallery'
    APPLICATION_VIEW_ICONS = 'icon-application-view-icons'
    APPLICATION_VIEW_LIST = 'icon-application-view-list'
    APPLICATION_VIEW_TILE = 'icon-application-view-tile'
    APPLICATION_XP = 'icon-application-xp'
    APPLICATION_XP_TERMINAL = 'icon-application-xp-terminal'
    ARROW_BRANCH = 'icon-arrow-branch'
    ARROW_DIVIDE = 'icon-arrow-divide'
    ARROW_DOWN = 'icon-arrow-down'
    ARROW_IN = 'icon-arrow-in'
    ARROW_INOUT = 'icon-arrow-inout'
    ARROW_JOIN = 'icon-arrow-join'
    ARROW_LEFT = 'icon-arrow-left'
    ARROW_MERGE = 'icon-arrow-merge'
    ARROW_OUT = 'icon-arrow-out'
    ARROW_REDO = 'icon-arrow-redo'
    ARROW_REFRESH = 'icon-arrow-refresh'
    ARROW_REFRESH_SMALL = 'icon-arrow-refresh-small'
    ARROW_RIGHT = 'icon-arrow-right'
    ARROW_ROTATE_ANTICLOCKWISE = 'icon-arrow-rotate-anticlockwise'
    ARROW_ROTATE_CLOCKWISE = 'icon-arrow-rotate-clockwise'
    ARROW_SWITCH = 'icon-arrow-switch'
    ARROW_TURN_LEFT = 'icon-arrow-turn-left'
    ARROW_TURN_RIGHT = 'icon-arrow-turn-right'
    ARROW_UNDO = 'icon-arrow-undo'
    ARROW_UP = 'icon-arrow-up'
    ASTERISK_ORANGE = 'icon-asterisk-orange'
    ASTERISK_YELLOW = 'icon-asterisk-yellow'
    ATTACH = 'icon-attach'
    AWARD_STAR_ADD = 'icon-award-star-add'
    AWARD_STAR_BRONZE_1 = 'icon-award-star-bronze-1'
    AWARD_STAR_BRONZE_2 = 'icon-award-star-bronze-2'
    AWARD_STAR_BRONZE_3 = 'icon-award-star-bronze-3'
    AWARD_STAR_DELETE = 'icon-award-star-delete'
    AWARD_STAR_GOLD_1 = 'icon-award-star-gold-1'
    AWARD_STAR_GOLD_2 = 'icon-award-star-gold-2'
    AWARD_STAR_GOLD_3 = 'icon-award-star-gold-3'
    AWARD_STAR_SILVER_1 = 'icon-award-star-silver-1'
    AWARD_STAR_SILVER_2 = 'icon-award-star-silver-2'
    AWARD_STAR_SILVER_3 = 'icon-award-star-silver-3'
    BASKET = 'icon-basket'
    BASKET_ADD = 'icon-basket-add'
    BASKET_DELETE = 'icon-basket-delete'
    BASKET_EDIT = 'icon-basket-edit'
    BASKET_ERROR = 'icon-basket-error'
    BASKET_GO = 'icon-basket-go'
    BASKET_PUT = 'icon-basket-put'
    BASKET_REMOVE = 'icon-basket-remove'
    BELL = 'icon-bell'
    BELL_ADD = 'icon-bell-add'
    BELL_DELETE = 'icon-bell-delete'
    BELL_ERROR = 'icon-bell-error'
    BELL_GO = 'icon-bell-go'
    BELL_LINK = 'icon-bell-link'
    BIN = 'icon-bin'
    BIN_CLOSED = 'icon-bin-closed'
    BIN_EMPTY = 'icon-bin-empty'
    BOMB = 'icon-bomb'
    BOOK = 'icon-book'
    BOOK_ADD = 'icon-book-add'
    BOOK_ADDRESSES = 'icon-book-addresses'
    BOOK_DELETE = 'icon-book-delete'
    BOOK_EDIT = 'icon-book-edit'
    BOOK_ERROR = 'icon-book-error'
    BOOK_GO = 'icon-book-go'
    BOOK_KEY = 'icon-book-key'
    BOOK_LINK = 'icon-book-link'
    BOOK_NEXT = 'icon-book-next'
    BOOK_OPEN = 'icon-book-open'
    BOOK_PREVIOUS = 'icon-book-previous'
    BOX = 'icon-box'
    BRICK = 'icon-brick'
    BRICKS = 'icon-bricks'
    BRICK_ADD = 'icon-brick-add'
    BRICK_DELETE = 'icon-brick-delete'
    BRICK_EDIT = 'icon-brick-edit'
    BRICK_ERROR = 'icon-brick-error'
    BRICK_GO = 'icon-brick-go'
    BRICK_LINK = 'icon-brick-link'
    BRIEFCASE = 'icon-briefcase'
    BUG = 'icon-bug'
    BUG_ADD = 'icon-bug-add'
    BUG_DELETE = 'icon-bug-delete'
    BUG_EDIT = 'icon-bug-edit'
    BUG_ERROR = 'icon-bug-error'
    BUG_GO = 'icon-bug-go'
    BUG_LINK = 'icon-bug-link'
    BUILDING = 'icon-building'
    BUILDING_ADD = 'icon-building-add'
    BUILDING_DELETE = 'icon-building-delete'
    BUILDING_EDIT = 'icon-building-edit'
    BUILDING_ERROR = 'icon-building-error'
    BUILDING_GO = 'icon-building-go'
    BUILDING_KEY = 'icon-building-key'
    BUILDING_LINK = 'icon-building-link'
    BULLET_ADD = 'icon-bullet-add'
    BULLET_ARROW_BOTTOM = 'icon-bullet-arrow-bottom'
    BULLET_ARROW_DOWN = 'icon-bullet-arrow-down'
    BULLET_ARROW_TOP = 'icon-bullet-arrow-top'
    BULLET_ARROW_UP = 'icon-bullet-arrow-up'
    BULLET_BLACK = 'icon-bullet-black'
    BULLET_BLUE = 'icon-bullet-blue'
    BULLET_DELETE = 'icon-bullet-delete'
    BULLET_DISK = 'icon-bullet-disk'
    BULLET_ERROR = 'icon-bullet-error'
    BULLET_FEED = 'icon-bullet-feed'
    BULLET_GO = 'icon-bullet-go'
    BULLET_GREEN = 'icon-bullet-green'
    BULLET_KEY = 'icon-bullet-key'
    BULLET_ORANGE = 'icon-bullet-orange'
    BULLET_PICTURE = 'icon-bullet-picture'
    BULLET_PINK = 'icon-bullet-pink'
    BULLET_PURPLE = 'icon-bullet-purple'
    BULLET_RED = 'icon-bullet-red'
    BULLET_STAR = 'icon-bullet-star'
    BULLET_TOGGLE_MINUS = 'icon-bullet-toggle-minus'
    BULLET_TOGGLE_PLUS = 'icon-bullet-toggle-plus'
    BULLET_WHITE = 'icon-bullet-white'
    BULLET_WRENCH = 'icon-bullet-wrench'
    BULLET_YELLOW = 'icon-bullet-yellow'
    CAKE = 'icon-cake'
    CALCULATOR = 'icon-calculator'
    CALCULATOR_ADD = 'icon-calculator-add'
    CALCULATOR_DELETE = 'icon-calculator-delete'
    CALCULATOR_EDIT = 'icon-calculator-edit'
    CALCULATOR_ERROR = 'icon-calculator-error'
    CALCULATOR_LINK = 'icon-calculator-link'
    CALENDAR = 'icon-calendar'
    CALENDAR_ADD = 'icon-calendar-add'
    CALENDAR_DELETE = 'icon-calendar-delete'
    CALENDAR_EDIT = 'icon-calendar-edit'
    CALENDAR_LINK = 'icon-calendar-link'
    CALENDAR_VIEW_DAY = 'icon-calendar-view-day'
    CALENDAR_VIEW_MONTH = 'icon-calendar-view-month'
    CALENDAR_VIEW_WEEK = 'icon-calendar-view-week'
    CAMERA = 'icon-camera'
    CAMERA_ADD = 'icon-camera-add'
    CAMERA_DELETE = 'icon-camera-delete'
    CAMERA_EDIT = 'icon-camera-edit'
    CAMERA_ERROR = 'icon-camera-error'
    CAMERA_GO = 'icon-camera-go'
    CAMERA_LINK = 'icon-camera-link'
    CAMERA_SMALL = 'icon-camera-small'
    CANCEL = 'icon-cancel'
    CAR = 'icon-car'
    CART = 'icon-cart'
    CART_ADD = 'icon-cart-add'
    CART_DELETE = 'icon-cart-delete'
    CART_EDIT = 'icon-cart-edit'
    CART_ERROR = 'icon-cart-error'
    CART_GO = 'icon-cart-go'
    CART_PUT = 'icon-cart-put'
    CART_REMOVE = 'icon-cart-remove'
    CAR_ADD = 'icon-car-add'
    CAR_DELETE = 'icon-car-delete'
    CD = 'icon-cd'
    CD_ADD = 'icon-cd-add'
    CD_BURN = 'icon-cd-burn'
    CD_DELETE = 'icon-cd-delete'
    CD_EDIT = 'icon-cd-edit'
    CD_EJECT = 'icon-cd-eject'
    CD_GO = 'icon-cd-go'
    CHART_BAR = 'icon-chart-bar'
    CHART_BAR_ADD = 'icon-chart-bar-add'
    CHART_BAR_DELETE = 'icon-chart-bar-delete'
    CHART_BAR_EDIT = 'icon-chart-bar-edit'
    CHART_BAR_ERROR = 'icon-chart-bar-error'
    CHART_BAR_LINK = 'icon-chart-bar-link'
    CHART_CURVE = 'icon-chart-curve'
    CHART_CURVE_ADD = 'icon-chart-curve-add'
    CHART_CURVE_DELETE = 'icon-chart-curve-delete'
    CHART_CURVE_EDIT = 'icon-chart-curve-edit'
    CHART_CURVE_ERROR = 'icon-chart-curve-error'
    CHART_CURVE_GO = 'icon-chart-curve-go'
    CHART_CURVE_LINK = 'icon-chart-curve-link'
    CHART_LINE = 'icon-chart-line'
    CHART_LINE_ADD = 'icon-chart-line-add'
    CHART_LINE_DELETE = 'icon-chart-line-delete'
    CHART_LINE_EDIT = 'icon-chart-line-edit'
    CHART_LINE_ERROR = 'icon-chart-line-error'
    CHART_LINE_LINK = 'icon-chart-line-link'
    CHART_ORGANISATION = 'icon-chart-organisation'
    CHART_ORGANISATION_ADD = 'icon-chart-organisation-add'
    CHART_ORGANISATION_DELETE = 'icon-chart-organisation-delete'
    CHART_PIE = 'icon-chart-pie'
    CHART_PIE_ADD = 'icon-chart-pie-add'
    CHART_PIE_DELETE = 'icon-chart-pie-delete'
    CHART_PIE_EDIT = 'icon-chart-pie-edit'
    CHART_PIE_ERROR = 'icon-chart-pie-error'
    CHART_PIE_LINK = 'icon-chart-pie-link'
    CLOCK = 'icon-clock'
    CLOCK_ADD = 'icon-clock-add'
    CLOCK_DELETE = 'icon-clock-delete'
    CLOCK_EDIT = 'icon-clock-edit'
    CLOCK_ERROR = 'icon-clock-error'
    CLOCK_GO = 'icon-clock-go'
    CLOCK_LINK = 'icon-clock-link'
    CLOCK_PAUSE = 'icon-clock-pause'
    CLOCK_PLAY = 'icon-clock-play'
    CLOCK_RED = 'icon-clock-red'
    CLOCK_STOP = 'icon-clock-stop'
    COG = 'icon-cog'
    COG_ADD = 'icon-cog-add'
    COG_DELETE = 'icon-cog-delete'
    COG_EDIT = 'icon-cog-edit'
    COG_ERROR = 'icon-cog-error'
    COG_GO = 'icon-cog-go'
    COINS = 'icon-coins'
    COINS_ADD = 'icon-coins-add'
    COINS_DELETE = 'icon-coins-delete'
    COLOR_SWATCH = 'icon-color-swatch'
    COLOR_WHEEL = 'icon-color-wheel'
    COMMENT = 'icon-comment'
    COMMENTS = 'icon-comments'
    COMMENTS_ADD = 'icon-comments-add'
    COMMENTS_DELETE = 'icon-comments-delete'
    COMMENT_ADD = 'icon-comment-add'
    COMMENT_DELETE = 'icon-comment-delete'
    COMMENT_EDIT = 'icon-comment-edit'
    COMPRESS = 'icon-compress'
    COMPUTER = 'icon-computer'
    COMPUTER_ADD = 'icon-computer-add'
    COMPUTER_DELETE = 'icon-computer-delete'
    COMPUTER_EDIT = 'icon-computer-edit'
    COMPUTER_ERROR = 'icon-computer-error'
    COMPUTER_GO = 'icon-computer-go'
    COMPUTER_KEY = 'icon-computer-key'
    COMPUTER_LINK = 'icon-computer-link'
    CONNECT = 'icon-connect'
    CONTRAST = 'icon-contrast'
    CONTRAST_DECREASE = 'icon-contrast-decrease'
    CONTRAST_HIGH = 'icon-contrast-high'
    CONTRAST_INCREASE = 'icon-contrast-increase'
    CONTRAST_LOW = 'icon-contrast-low'
    CONTROLLER = 'icon-controller'
    CONTROLLER_ADD = 'icon-controller-add'
    CONTROLLER_DELETE = 'icon-controller-delete'
    CONTROLLER_ERROR = 'icon-controller-error'
    CONTROL_EJECT = 'icon-control-eject'
    CONTROL_EJECT_BLUE = 'icon-control-eject-blue'
    CONTROL_END = 'icon-control-end'
    CONTROL_END_BLUE = 'icon-control-end-blue'
    CONTROL_EQUALIZER = 'icon-control-equalizer'
    CONTROL_EQUALIZER_BLUE = 'icon-control-equalizer-blue'
    CONTROL_FASTFORWARD = 'icon-control-fastforward'
    CONTROL_FASTFORWARD_BLUE = 'icon-control-fastforward-blue'
    CONTROL_PAUSE = 'icon-control-pause'
    CONTROL_PAUSE_BLUE = 'icon-control-pause-blue'
    CONTROL_PLAY = 'icon-control-play'
    CONTROL_PLAY_BLUE = 'icon-control-play-blue'
    CONTROL_REPEAT = 'icon-control-repeat'
    CONTROL_REPEAT_BLUE = 'icon-control-repeat-blue'
    CONTROL_REWIND = 'icon-control-rewind'
    CONTROL_REWIND_BLUE = 'icon-control-rewind-blue'
    CONTROL_START = 'icon-control-start'
    CONTROL_START_BLUE = 'icon-control-start-blue'
    CONTROL_STOP = 'icon-control-stop'
    CONTROL_STOP_BLUE = 'icon-control-stop-blue'
    CREDITCARDS = 'icon-creditcards'
    CROSS = 'icon-cross'
    CSS = 'icon-css'
    CSS_ADD = 'icon-css-add'
    CSS_DELETE = 'icon-css-delete'
    CSS_GO = 'icon-css-go'
    CSS_VALID = 'icon-css-valid'
    CUP = 'icon-cup'
    CUP_ADD = 'icon-cup-add'
    CUP_DELETE = 'icon-cup-delete'
    CUP_EDIT = 'icon-cup-edit'
    CUP_ERROR = 'icon-cup-error'
    CUP_GO = 'icon-cup-go'
    CUP_KEY = 'icon-cup-key'
    CUP_LINK = 'icon-cup-link'
    CURSOR = 'icon-cursor'
    CUT = 'icon-cut'
    CUT_RED = 'icon-cut-red'
    DATABASE = 'icon-database'
    DATABASE_ADD = 'icon-database-add'
    DATABASE_CONNECT = 'icon-database-connect'
    DATABASE_DELETE = 'icon-database-delete'
    DATABASE_EDIT = 'icon-database-edit'
    DATABASE_ERROR = 'icon-database-error'
    DATABASE_GEAR = 'icon-database-gear'
    DATABASE_GO = 'icon-database-go'
    DATABASE_KEY = 'icon-database-key'
    DATABASE_LIGHTNING = 'icon-database-lightning'
    DATABASE_LINK = 'icon-database-link'
    DATABASE_REFRESH = 'icon-database-refresh'
    DATABASE_SAVE = 'icon-database-save'
    DATABASE_TABLE = 'icon-database-table'
    DATE = 'icon-date'
    DATE_ADD = 'icon-date-add'
    DATE_DELETE = 'icon-date-delete'
    DATE_EDIT = 'icon-date-edit'
    DATE_ERROR = 'icon-date-error'
    DATE_GO = 'icon-date-go'
    DATE_LINK = 'icon-date-link'
    DATE_MAGNIFY = 'icon-date-magnify'
    DATE_NEXT = 'icon-date-next'
    DATE_PREVIOUS = 'icon-date-previous'
    DELETE = 'icon-delete'
    DISCONNECT = 'icon-disconnect'
    DISK = 'icon-disk'
    DISK_MULTIPLE = 'icon-disk-multiple'
    DOOR = 'icon-door'
    DOOR_IN = 'icon-door-in'
    DOOR_OPEN = 'icon-door-open'
    DOOR_OUT = 'icon-door-out'
    DRINK = 'icon-drink'
    DRINK_EMPTY = 'icon-drink-empty'
    DRIVE = 'icon-drive'
    DRIVE_ADD = 'icon-drive-add'
    DRIVE_BURN = 'icon-drive-burn'
    DRIVE_CD = 'icon-drive-cd'
    DRIVE_CD_EMPTY = 'icon-drive-cd-empty'
    DRIVE_DELETE = 'icon-drive-delete'
    DRIVE_DISK = 'icon-drive-disk'
    DRIVE_EDIT = 'icon-drive-edit'
    DRIVE_ERROR = 'icon-drive-error'
    DRIVE_GO = 'icon-drive-go'
    DRIVE_KEY = 'icon-drive-key'
    DRIVE_LINK = 'icon-drive-link'
    DRIVE_MAGNIFY = 'icon-drive-magnify'
    DRIVE_NETWORK = 'icon-drive-network'
    DRIVE_RENAME = 'icon-drive-rename'
    DRIVE_USER = 'icon-drive-user'
    DRIVE_WEB = 'icon-drive-web'
    DVD = 'icon-dvd'
    DVD_ADD = 'icon-dvd-add'
    DVD_DELETE = 'icon-dvd-delete'
    DVD_EDIT = 'icon-dvd-edit'
    DVD_ERROR = 'icon-dvd-error'
    DVD_GO = 'icon-dvd-go'
    DVD_KEY = 'icon-dvd-key'
    DVD_LINK = 'icon-dvd-link'
    EMAIL = 'icon-email'
    EMAIL_ADD = 'icon-email-add'
    EMAIL_ATTACH = 'icon-email-attach'
    EMAIL_DELETE = 'icon-email-delete'
    EMAIL_EDIT = 'icon-email-edit'
    EMAIL_ERROR = 'icon-email-error'
    EMAIL_GO = 'icon-email-go'
    EMAIL_LINK = 'icon-email-link'
    EMAIL_OPEN = 'icon-email-open'
    EMAIL_OPEN_IMAGE = 'icon-email-open-image'
    EMOTICON_EVILGRIN = 'icon-emoticon-evilgrin'
    EMOTICON_GRIN = 'icon-emoticon-grin'
    EMOTICON_HAPPY = 'icon-emoticon-happy'
    EMOTICON_SMILE = 'icon-emoticon-smile'
    EMOTICON_SURPRISED = 'icon-emoticon-surprised'
    EMOTICON_TONGUE = 'icon-emoticon-tongue'
    EMOTICON_UNHAPPY = 'icon-emoticon-unhappy'
    EMOTICON_WAII = 'icon-emoticon-waii'
    EMOTICON_WINK = 'icon-emoticon-wink'
    ERROR = 'icon-error'
    ERROR_ADD = 'icon-error-add'
    ERROR_DELETE = 'icon-error-delete'
    ERROR_GO = 'icon-error-go'
    EXCLAMATION = 'icon-exclamation'
    EYE = 'icon-eye'
    FEED = 'icon-feed'
    FEED_ADD = 'icon-feed-add'
    FEED_DELETE = 'icon-feed-delete'
    FEED_DISK = 'icon-feed-disk'
    FEED_EDIT = 'icon-feed-edit'
    FEED_ERROR = 'icon-feed-error'
    FEED_GO = 'icon-feed-go'
    FEED_KEY = 'icon-feed-key'
    FEED_LINK = 'icon-feed-link'
    FEED_MAGNIFY = 'icon-feed-magnify'
    FEMALE = 'icon-female'
    FILM = 'icon-film'
    FILM_ADD = 'icon-film-add'
    FILM_DELETE = 'icon-film-delete'
    FILM_EDIT = 'icon-film-edit'
    FILM_ERROR = 'icon-film-error'
    FILM_GO = 'icon-film-go'
    FILM_KEY = 'icon-film-key'
    FILM_LINK = 'icon-film-link'
    FILM_SAVE = 'icon-film-save'
    FIND = 'icon-find'
    FLAG_BLUE = 'icon-flag-blue'
    FLAG_GREEN = 'icon-flag-green'
    FLAG_ORANGE = 'icon-flag-orange'
    FLAG_PINK = 'icon-flag-pink'
    FLAG_PURPLE = 'icon-flag-purple'
    FLAG_RED = 'icon-flag-red'
    FLAG_YELLOW = 'icon-flag-yellow'
    FOLDER = 'icon-folder'
    FOLDER_ADD = 'icon-folder-add'
    FOLDER_BELL = 'icon-folder-bell'
    FOLDER_BRICK = 'icon-folder-brick'
    FOLDER_BUG = 'icon-folder-bug'
    FOLDER_CAMERA = 'icon-folder-camera'
    FOLDER_DATABASE = 'icon-folder-database'
    FOLDER_DELETE = 'icon-folder-delete'
    FOLDER_EDIT = 'icon-folder-edit'
    FOLDER_ERROR = 'icon-folder-error'
    FOLDER_EXPLORE = 'icon-folder-explore'
    FOLDER_FEED = 'icon-folder-feed'
    FOLDER_FIND = 'icon-folder-find'
    FOLDER_GO = 'icon-folder-go'
    FOLDER_HEART = 'icon-folder-heart'
    FOLDER_IMAGE = 'icon-folder-image'
    FOLDER_KEY = 'icon-folder-key'
    FOLDER_LIGHTBULB = 'icon-folder-lightbulb'
    FOLDER_LINK = 'icon-folder-link'
    FOLDER_MAGNIFY = 'icon-folder-magnify'
    FOLDER_PAGE = 'icon-folder-page'
    FOLDER_PAGE_WHITE = 'icon-folder-page-white'
    FOLDER_PALETTE = 'icon-folder-palette'
    FOLDER_PICTURE = 'icon-folder-picture'
    FOLDER_STAR = 'icon-folder-star'
    FOLDER_TABLE = 'icon-folder-table'
    FOLDER_USER = 'icon-folder-user'
    FOLDER_WRENCH = 'icon-folder-wrench'
    FONT = 'icon-font'
    FONT_ADD = 'icon-font-add'
    FONT_DELETE = 'icon-font-delete'
    FONT_GO = 'icon-font-go'
    GROUP = 'icon-group'
    GROUP_ADD = 'icon-group-add'
    GROUP_DELETE = 'icon-group-delete'
    GROUP_EDIT = 'icon-group-edit'
    GROUP_ERROR = 'icon-group-error'
    GROUP_GEAR = 'icon-group-gear'
    GROUP_GO = 'icon-group-go'
    GROUP_KEY = 'icon-group-key'
    GROUP_LINK = 'icon-group-link'
    HEART = 'icon-heart'
    HEART_ADD = 'icon-heart-add'
    HEART_DELETE = 'icon-heart-delete'
    HELP = 'icon-help'
    HOURGLASS = 'icon-hourglass'
    HOURGLASS_ADD = 'icon-hourglass-add'
    HOURGLASS_DELETE = 'icon-hourglass-delete'
    HOURGLASS_GO = 'icon-hourglass-go'
    HOURGLASS_LINK = 'icon-hourglass-link'
    HOUSE = 'icon-house'
    HOUSE_GO = 'icon-house-go'
    HOUSE_LINK = 'icon-house-link'
    HTML = 'icon-html'
    HTML_ADD = 'icon-html-add'
    HTML_DELETE = 'icon-html-delete'
    HTML_GO = 'icon-html-go'
    HTML_VALID = 'icon-html-valid'
    IMAGE = 'icon-image'
    IMAGES = 'icon-images'
    IMAGE_ADD = 'icon-image-add'
    IMAGE_DELETE = 'icon-image-delete'
    IMAGE_EDIT = 'icon-image-edit'
    IMAGE_LINK = 'icon-image-link'
    INFORMATION = 'icon-information'
    IPOD = 'icon-ipod'
    IPOD_CAST = 'icon-ipod-cast'
    IPOD_CAST_ADD = 'icon-ipod-cast-add'
    IPOD_CAST_DELETE = 'icon-ipod-cast-delete'
    IPOD_SOUND = 'icon-ipod-sound'
    JOYSTICK = 'icon-joystick'
    JOYSTICK_ADD = 'icon-joystick-add'
    JOYSTICK_DELETE = 'icon-joystick-delete'
    JOYSTICK_ERROR = 'icon-joystick-error'
    KEY = 'icon-key'
    KEYBOARD = 'icon-keyboard'
    KEYBOARD_ADD = 'icon-keyboard-add'
    KEYBOARD_DELETE = 'icon-keyboard-delete'
    KEYBOARD_MAGNIFY = 'icon-keyboard-magnify'
    KEY_ADD = 'icon-key-add'
    KEY_DELETE = 'icon-key-delete'
    KEY_GO = 'icon-key-go'
    LAYERS = 'icon-layers'
    LAYOUT = 'icon-layout'
    LAYOUT_ADD = 'icon-layout-add'
    LAYOUT_CONTENT = 'icon-layout-content'
    LAYOUT_DELETE = 'icon-layout-delete'
    LAYOUT_EDIT = 'icon-layout-edit'
    LAYOUT_ERROR = 'icon-layout-error'
    LAYOUT_HEADER = 'icon-layout-header'
    LAYOUT_LINK = 'icon-layout-link'
    LAYOUT_SIDEBAR = 'icon-layout-sidebar'
    LIGHTBULB = 'icon-lightbulb'
    LIGHTBULB_ADD = 'icon-lightbulb-add'
    LIGHTBULB_DELETE = 'icon-lightbulb-delete'
    LIGHTBULB_OFF = 'icon-lightbulb-off'
    LIGHTNING = 'icon-lightning'
    LIGHTNING_ADD = 'icon-lightning-add'
    LIGHTNING_DELETE = 'icon-lightning-delete'
    LIGHTNING_GO = 'icon-lightning-go'
    LINK = 'icon-link'
    LINK_ADD = 'icon-link-add'
    LINK_BREAK = 'icon-link-break'
    LINK_DELETE = 'icon-link-delete'
    LINK_EDIT = 'icon-link-edit'
    LINK_ERROR = 'icon-link-error'
    LINK_GO = 'icon-link-go'
    LOCK = 'icon-lock'
    LOCK_ADD = 'icon-lock-add'
    LOCK_BREAK = 'icon-lock-break'
    LOCK_DELETE = 'icon-lock-delete'
    LOCK_EDIT = 'icon-lock-edit'
    LOCK_GO = 'icon-lock-go'
    LOCK_OPEN = 'icon-lock-open'
    LORRY = 'icon-lorry'
    LORRY_ADD = 'icon-lorry-add'
    LORRY_DELETE = 'icon-lorry-delete'
    LORRY_ERROR = 'icon-lorry-error'
    LORRY_FLATBED = 'icon-lorry-flatbed'
    LORRY_GO = 'icon-lorry-go'
    LORRY_LINK = 'icon-lorry-link'
    MAGIFIER_ZOOM_OUT = 'icon-magifier-zoom-out'
    MAGNIFIER = 'icon-magnifier'
    MAGNIFIER_ZOOM_IN = 'icon-magnifier-zoom-in'
    MALE = 'icon-male'
    MAP = 'icon-map'
    MAP_ADD = 'icon-map-add'
    MAP_DELETE = 'icon-map-delete'
    MAP_EDIT = 'icon-map-edit'
    MAP_GO = 'icon-map-go'
    MAP_MAGNIFY = 'icon-map-magnify'
    MEDAL_BRONZE_1 = 'icon-medal-bronze-1'
    MEDAL_BRONZE_2 = 'icon-medal-bronze-2'
    MEDAL_BRONZE_3 = 'icon-medal-bronze-3'
    MEDAL_BRONZE_ADD = 'icon-medal-bronze-add'
    MEDAL_BRONZE_DELETE = 'icon-medal-bronze-delete'
    MEDAL_GOLD_1 = 'icon-medal-gold-1'
    MEDAL_GOLD_2 = 'icon-medal-gold-2'
    MEDAL_GOLD_3 = 'icon-medal-gold-3'
    MEDAL_GOLD_ADD = 'icon-medal-gold-add'
    MEDAL_GOLD_DELETE = 'icon-medal-gold-delete'
    MEDAL_SILVER_1 = 'icon-medal-silver-1'
    MEDAL_SILVER_2 = 'icon-medal-silver-2'
    MEDAL_SILVER_3 = 'icon-medal-silver-3'
    MEDAL_SILVER_ADD = 'icon-medal-silver-add'
    MEDAL_SILVER_DELETE = 'icon-medal-silver-delete'
    MONEY = 'icon-money'
    MONEY_ADD = 'icon-money-add'
    MONEY_DELETE = 'icon-money-delete'
    MONEY_DOLLAR = 'icon-money-dollar'
    MONEY_EURO = 'icon-money-euro'
    MONEY_POUND = 'icon-money-pound'
    MONEY_YEN = 'icon-money-yen'
    MONITOR = 'icon-monitor'
    MONITOR_ADD = 'icon-monitor-add'
    MONITOR_DELETE = 'icon-monitor-delete'
    MONITOR_EDIT = 'icon-monitor-edit'
    MONITOR_ERROR = 'icon-monitor-error'
    MONITOR_GO = 'icon-monitor-go'
    MONITOR_LIGHTNING = 'icon-monitor-lightning'
    MONITOR_LINK = 'icon-monitor-link'
    MOUSE = 'icon-mouse'
    MOUSE_ADD = 'icon-mouse-add'
    MOUSE_DELETE = 'icon-mouse-delete'
    MOUSE_ERROR = 'icon-mouse-error'
    MUSIC = 'icon-music'
    NEW = 'icon-new'
    NEWSPAPER = 'icon-newspaper'
    NEWSPAPER_ADD = 'icon-newspaper-add'
    NEWSPAPER_DELETE = 'icon-newspaper-delete'
    NEWSPAPER_GO = 'icon-newspaper-go'
    NEWSPAPER_LINK = 'icon-newspaper-link'
    NOTE = 'icon-note'
    NOTE_ADD = 'icon-note-add'
    NOTE_DELETE = 'icon-note-delete'
    NOTE_EDIT = 'icon-note-edit'
    NOTE_ERROR = 'icon-note-error'
    NOTE_GO = 'icon-note-go'
    OVERLAYS = 'icon-overlays'
    PACKAGE = 'icon-package'
    PACKAGE_PY = 'icon-package-py'
    PACKAGE_ADD = 'icon-package-add'
    PACKAGE_DELETE = 'icon-package-delete'
    PACKAGE_GO = 'icon-package-go'
    PACKAGE_GREEN = 'icon-package-green'
    PACKAGE_LINK = 'icon-package-link'
    PAGE = 'icon-page'
    PAGE_ADD = 'icon-page-add'
    PAGE_ATTACH = 'icon-page-attach'
    PAGE_CODE = 'icon-page-code'
    PAGE_COPY = 'icon-page-copy'
    PAGE_DELETE = 'icon-page-delete'
    PAGE_EDIT = 'icon-page-edit'
    PAGE_ERROR = 'icon-page-error'
    PAGE_EXCEL = 'icon-page-excel'
    PAGE_FIND = 'icon-page-find'
    PAGE_GEAR = 'icon-page-gear'
    PAGE_GO = 'icon-page-go'
    PAGE_GREEN = 'icon-page-green'
    PAGE_KEY = 'icon-page-key'
    PAGE_LIGHTNING = 'icon-page-lightning'
    PAGE_LINK = 'icon-page-link'
    PAGE_PAINTBRUSH = 'icon-page-paintbrush'
    PAGE_PASTE = 'icon-page-paste'
    PAGE_RED = 'icon-page-red'
    PAGE_REFRESH = 'icon-page-refresh'
    PAGE_SAVE = 'icon-page-save'
    PAGE_WHITE = 'icon-page-white'
    PAGE_WHITE_ACROBAT = 'icon-page-white-acrobat'
    PAGE_WHITE_ACTIONSCRIPT = 'icon-page-white-actionscript'
    PAGE_WHITE_ADD = 'icon-page-white-add'
    PAGE_WHITE_C = 'icon-page-white-c'
    PAGE_WHITE_CAMERA = 'icon-page-white-camera'
    PAGE_WHITE_CD = 'icon-page-white-cd'
    PAGE_WHITE_CODE = 'icon-page-white-code'
    PAGE_WHITE_CODE_RED = 'icon-page-white-code-red'
    PAGE_WHITE_COLDFUSION = 'icon-page-white-coldfusion'
    PAGE_WHITE_COMPRESSED = 'icon-page-white-compressed'
    PAGE_WHITE_COPY = 'icon-page-white-copy'
    PAGE_WHITE_CPLUSPLUS = 'icon-page-white-cplusplus'
    PAGE_WHITE_CSHARP = 'icon-page-white-csharp'
    PAGE_WHITE_CUP = 'icon-page-white-cup'
    PAGE_WHITE_DATABASE = 'icon-page-white-database'
    PAGE_WHITE_DELETE = 'icon-page-white-delete'
    PAGE_WHITE_DVD = 'icon-page-white-dvd'
    PAGE_WHITE_EDIT = 'icon-page-white-edit'
    PAGE_WHITE_ERROR = 'icon-page-white-error'
    PAGE_WHITE_EXCEL = 'icon-page-white-excel'
    PAGE_WHITE_FIND = 'icon-page-white-find'
    PAGE_WHITE_FLASH = 'icon-page-white-flash'
    PAGE_WHITE_FREEHAND = 'icon-page-white-freehand'
    PAGE_WHITE_GEAR = 'icon-page-white-gear'
    PAGE_WHITE_GET = 'icon-page-white-get'
    PAGE_WHITE_GO = 'icon-page-white-go'
    PAGE_WHITE_H = 'icon-page-white-h'
    PAGE_WHITE_HORIZONTAL = 'icon-page-white-horizontal'
    PAGE_WHITE_KEY = 'icon-page-white-key'
    PAGE_WHITE_LIGHTNING = 'icon-page-white-lightning'
    PAGE_WHITE_LINK = 'icon-page-white-link'
    PAGE_WHITE_MAGNIFY = 'icon-page-white-magnify'
    PAGE_WHITE_MEDAL = 'icon-page-white-medal'
    PAGE_WHITE_OFFICE = 'icon-page-white-office'
    PAGE_WHITE_PAINT = 'icon-page-white-paint'
    PAGE_WHITE_PAINTBRUSH = 'icon-page-white-paintbrush'
    PAGE_WHITE_PASTE = 'icon-page-white-paste'
    PAGE_WHITE_PHP = 'icon-page-white-php'
    PAGE_WHITE_PICTURE = 'icon-page-white-picture'
    PAGE_WHITE_POWERPOINT = 'icon-page-white-powerpoint'
    PAGE_WHITE_PUT = 'icon-page-white-put'
    PAGE_WHITE_PY = 'icon-page-white-py'
    PAGE_WHITE_JS = 'icon-page-white-js'
    PAGE_WHITE_RUBY = 'icon-page-white-ruby'
    PAGE_WHITE_STACK = 'icon-page-white-stack'
    PAGE_WHITE_STAR = 'icon-page-white-star'
    PAGE_WHITE_SWOOSH = 'icon-page-white-swoosh'
    PAGE_WHITE_TEXT = 'icon-page-white-text'
    PAGE_WHITE_TEXT_WIDTH = 'icon-page-white-text-width'
    PAGE_WHITE_TUX = 'icon-page-white-tux'
    PAGE_WHITE_VECTOR = 'icon-page-white-vector'
    PAGE_WHITE_VISUALSTUDIO = 'icon-page-white-visualstudio'
    PAGE_WHITE_WIDTH = 'icon-page-white-width'
    PAGE_WHITE_WORD = 'icon-page-white-word'
    PAGE_WHITE_WORLD = 'icon-page-white-world'
    PAGE_WHITE_WRENCH = 'icon-page-white-wrench'
    PAGE_WHITE_ZIP = 'icon-page-white-zip'
    PAGE_WORD = 'icon-page-word'
    PAGE_WORLD = 'icon-page-world'
    PAINTBRUSH = 'icon-paintbrush'
    PAINTCAN = 'icon-paintcan'
    PALETTE = 'icon-palette'
    PASTE_PLAIN = 'icon-paste-plain'
    PASTE_WORD = 'icon-paste-word'
    PENCIL = 'icon-pencil'
    PENCIL_ADD = 'icon-pencil-add'
    PENCIL_DELETE = 'icon-pencil-delete'
    PENCIL_GO = 'icon-pencil-go'
    PHONE = 'icon-phone'
    PHONE_ADD = 'icon-phone-add'
    PHONE_DELETE = 'icon-phone-delete'
    PHONE_SOUND = 'icon-phone-sound'
    PHOTO = 'icon-photo'
    PHOTOS = 'icon-photos'
    PHOTO_ADD = 'icon-photo-add'
    PHOTO_DELETE = 'icon-photo-delete'
    PHOTO_LINK = 'icon-photo-link'
    PICTURE = 'icon-picture'
    PICTURES = 'icon-pictures'
    PICTURE_ADD = 'icon-picture-add'
    PICTURE_DELETE = 'icon-picture-delete'
    PICTURE_EDIT = 'icon-picture-edit'
    PICTURE_EMPTY = 'icon-picture-empty'
    PICTURE_ERROR = 'icon-picture-error'
    PICTURE_GO = 'icon-picture-go'
    PICTURE_KEY = 'icon-picture-key'
    PICTURE_LINK = 'icon-picture-link'
    PICTURE_SAVE = 'icon-picture-save'
    PILCROW = 'icon-pilcrow'
    PILL = 'icon-pill'
    PILL_ADD = 'icon-pill-add'
    PILL_DELETE = 'icon-pill-delete'
    PILL_GO = 'icon-pill-go'
    PLUGIN = 'icon-plugin'
    PLUGIN_ADD = 'icon-plugin-add'
    PLUGIN_DELETE = 'icon-plugin-delete'
    PLUGIN_DISABLED = 'icon-plugin-disabled'
    PLUGIN_EDIT = 'icon-plugin-edit'
    PLUGIN_ERROR = 'icon-plugin-error'
    PLUGIN_GO = 'icon-plugin-go'
    PLUGIN_LINK = 'icon-plugin-link'
    PRINTER = 'icon-printer'
    PRINTER_ADD = 'icon-printer-add'
    PRINTER_DELETE = 'icon-printer-delete'
    PRINTER_EMPTY = 'icon-printer-empty'
    PRINTER_ERROR = 'icon-printer-error'
    RAINBOW = 'icon-rainbow'
    REPORT = 'icon-report'
    REPORT_ADD = 'icon-report-add'
    REPORT_DELETE = 'icon-report-delete'
    REPORT_DISK = 'icon-report-disk'
    REPORT_EDIT = 'icon-report-edit'
    REPORT_GO = 'icon-report-go'
    REPORT_KEY = 'icon-report-key'
    REPORT_LINK = 'icon-report-link'
    REPORT_MAGNIFY = 'icon-report-magnify'
    REPORT_PICTURE = 'icon-report-picture'
    REPORT_USER = 'icon-report-user'
    REPORT_WORD = 'icon-report-word'
    RESULTSET_FIRST = 'icon-resultset-first'
    RESULTSET_LAST = 'icon-resultset-last'
    RESULTSET_NEXT = 'icon-resultset-next'
    RESULTSET_PREVIOUS = 'icon-resultset-previous'
    ROSETTE = 'icon-rosette'
    RSS = 'icon-rss'
    RSS_ADD = 'icon-rss-add'
    RSS_DELETE = 'icon-rss-delete'
    RSS_GO = 'icon-rss-go'
    RSS_VALID = 'icon-rss-valid'
    RUBY = 'icon-ruby'
    RUBY_ADD = 'icon-ruby-add'
    RUBY_DELETE = 'icon-ruby-delete'
    RUBY_GEAR = 'icon-ruby-gear'
    RUBY_GET = 'icon-ruby-get'
    RUBY_GO = 'icon-ruby-go'
    RUBY_KEY = 'icon-ruby-key'
    RUBY_LINK = 'icon-ruby-link'
    RUBY_PUT = 'icon-ruby-put'
    SCRIPT = 'icon-script'
    SCRIPT_ADD = 'icon-script-add'
    SCRIPT_CODE = 'icon-script-code'
    SCRIPT_CODE_RED = 'icon-script-code-red'
    SCRIPT_DELETE = 'icon-script-delete'
    SCRIPT_EDIT = 'icon-script-edit'
    SCRIPT_ERROR = 'icon-script-error'
    SCRIPT_GEAR = 'icon-script-gear'
    SCRIPT_GO = 'icon-script-go'
    SCRIPT_KEY = 'icon-script-key'
    SCRIPT_LIGHTNING = 'icon-script-lightning'
    SCRIPT_LINK = 'icon-script-link'
    SCRIPT_PALETTE = 'icon-script-palette'
    SCRIPT_SAVE = 'icon-script-save'
    SERVER = 'icon-server'
    SERVER_ADD = 'icon-server-add'
    SERVER_CHART = 'icon-server-chart'
    SERVER_COMPRESSED = 'icon-server-compressed'
    SERVER_CONNECT = 'icon-server-connect'
    SERVER_DATABASE = 'icon-server-database'
    SERVER_DELETE = 'icon-server-delete'
    SERVER_EDIT = 'icon-server-edit'
    SERVER_ERROR = 'icon-server-error'
    SERVER_GO = 'icon-server-go'
    SERVER_KEY = 'icon-server-key'
    SERVER_LIGHTNING = 'icon-server-lightning'
    SERVER_LINK = 'icon-server-link'
    SERVER_UNCOMPRESSED = 'icon-server-uncompressed'
    SHADING = 'icon-shading'
    SHAPE_ALIGN_BOTTOM = 'icon-shape-align-bottom'
    SHAPE_ALIGN_CENTER = 'icon-shape-align-center'
    SHAPE_ALIGN_LEFT = 'icon-shape-align-left'
    SHAPE_ALIGN_MIDDLE = 'icon-shape-align-middle'
    SHAPE_ALIGN_RIGHT = 'icon-shape-align-right'
    SHAPE_ALIGN_TOP = 'icon-shape-align-top'
    SHAPE_FLIP_HORIZONTAL = 'icon-shape-flip-horizontal'
    SHAPE_FLIP_VERTICAL = 'icon-shape-flip-vertical'
    SHAPE_GROUP = 'icon-shape-group'
    SHAPE_HANDLES = 'icon-shape-handles'
    SHAPE_MOVE_BACK = 'icon-shape-move-back'
    SHAPE_MOVE_BACKWARDS = 'icon-shape-move-backwards'
    SHAPE_MOVE_FORWARDS = 'icon-shape-move-forwards'
    SHAPE_MOVE_FRONT = 'icon-shape-move-front'
    SHAPE_ROTATE_ANTICLOCKWISE = 'icon-shape-rotate-anticlockwise'
    SHAPE_ROTATE_CLOCKWISE = 'icon-shape-rotate-clockwise'
    SHAPE_SQUARE = 'icon-shape-square'
    SHAPE_SQUARE_ADD = 'icon-shape-square-add'
    SHAPE_SQUARE_DELETE = 'icon-shape-square-delete'
    SHAPE_SQUARE_EDIT = 'icon-shape-square-edit'
    SHAPE_SQUARE_ERROR = 'icon-shape-square-error'
    SHAPE_SQUARE_GO = 'icon-shape-square-go'
    SHAPE_SQUARE_KEY = 'icon-shape-square-key'
    SHAPE_SQUARE_LINK = 'icon-shape-square-link'
    SHAPE_UNGROUP = 'icon-shape-ungroup'
    SHIELD = 'icon-shield'
    SHIELD_ADD = 'icon-shield-add'
    SHIELD_DELETE = 'icon-shield-delete'
    SHIELD_GO = 'icon-shield-go'
    SITEMAP = 'icon-sitemap'
    SITEMAP_COLOR = 'icon-sitemap-color'
    SOUND = 'icon-sound'
    SOUND_ADD = 'icon-sound-add'
    SOUND_DELETE = 'icon-sound-delete'
    SOUND_LOW = 'icon-sound-low'
    SOUND_MUTE = 'icon-sound-mute'
    SOUND_NONE = 'icon-sound-none'
    SPELLCHECK = 'icon-spellcheck'
    SPORT_8BALL = 'icon-sport-8ball'
    SPORT_BASKETBALL = 'icon-sport-basketball'
    SPORT_FOOTBALL = 'icon-sport-football'
    SPORT_GOLF = 'icon-sport-golf'
    SPORT_RAQUET = 'icon-sport-raquet'
    SPORT_SHUTTLECOCK = 'icon-sport-shuttlecock'
    SPORT_SOCCER = 'icon-sport-soccer'
    SPORT_TENNIS = 'icon-sport-tennis'
    STAR = 'icon-star'
    STATUS_AWAY = 'icon-status-away'
    STATUS_BUSY = 'icon-status-busy'
    STATUS_OFFLINE = 'icon-status-offline'
    STATUS_ONLINE = 'icon-status-online'
    STOP = 'icon-stop'
    STYLE = 'icon-style'
    STYLE_ADD = 'icon-style-add'
    STYLE_DELETE = 'icon-style-delete'
    STYLE_EDIT = 'icon-style-edit'
    STYLE_GO = 'icon-style-go'
    SUM = 'icon-sum'
    TAB = 'icon-tab'
    TABLE = 'icon-table'
    TABLE_ADD = 'icon-table-add'
    TABLE_DELETE = 'icon-table-delete'
    TABLE_EDIT = 'icon-table-edit'
    TABLE_ERROR = 'icon-table-error'
    TABLE_GEAR = 'icon-table-gear'
    TABLE_GO = 'icon-table-go'
    TABLE_KEY = 'icon-table-key'
    TABLE_LIGHTNING = 'icon-table-lightning'
    TABLE_LINK = 'icon-table-link'
    TABLE_MULTIPLE = 'icon-table-multiple'
    TABLE_REFRESH = 'icon-table-refresh'
    TABLE_RELATIONSHIP = 'icon-table-relationship'
    TABLE_ROW_DELETE = 'icon-table-row-delete'
    TABLE_ROW_INSERT = 'icon-table-row-insert'
    TABLE_SAVE = 'icon-table-save'
    TABLE_SORT = 'icon-table-sort'
    TAB_ADD = 'icon-tab-add'
    TAB_DELETE = 'icon-tab-delete'
    TAB_EDIT = 'icon-tab-edit'
    TAB_GO = 'icon-tab-go'
    TAG = 'icon-tag'
    TAG_BLUE = 'icon-tag-blue'
    TAG_BLUE_ADD = 'icon-tag-blue-add'
    TAG_BLUE_DELETE = 'icon-tag-blue-delete'
    TAG_BLUE_EDIT = 'icon-tag-blue-edit'
    TAG_GREEN = 'icon-tag-green'
    TAG_ORANGE = 'icon-tag-orange'
    TAG_PINK = 'icon-tag-pink'
    TAG_PURPLE = 'icon-tag-purple'
    TAG_RED = 'icon-tag-red'
    TAG_YELLOW = 'icon-tag-yellow'
    TELEPHONE = 'icon-telephone'
    TELEPHONE_ADD = 'icon-telephone-add'
    TELEPHONE_DELETE = 'icon-telephone-delete'
    TELEPHONE_EDIT = 'icon-telephone-edit'
    TELEPHONE_ERROR = 'icon-telephone-error'
    TELEPHONE_GO = 'icon-telephone-go'
    TELEPHONE_KEY = 'icon-telephone-key'
    TELEPHONE_LINK = 'icon-telephone-link'
    TELEVISION = 'icon-television'
    TELEVISION_ADD = 'icon-television-add'
    TELEVISION_DELETE = 'icon-television-delete'
    TEXTFIELD = 'icon-textfield'
    TEXTFIELD_ADD = 'icon-textfield-add'
    TEXTFIELD_DELETE = 'icon-textfield-delete'
    TEXTFIELD_KEY = 'icon-textfield-key'
    TEXTFIELD_RENAME = 'icon-textfield-rename'
    TEXT_ALIGN_CENTER = 'icon-text-align-center'
    TEXT_ALIGN_JUSTIFY = 'icon-text-align-justify'
    TEXT_ALIGN_LEFT = 'icon-text-align-left'
    TEXT_ALIGN_RIGHT = 'icon-text-align-right'
    TEXT_ALLCAPS = 'icon-text-allcaps'
    TEXT_BOLD = 'icon-text-bold'
    TEXT_COLUMNS = 'icon-text-columns'
    TEXT_DROPCAPS = 'icon-text-dropcaps'
    TEXT_HEADING_1 = 'icon-text-heading-1'
    TEXT_HEADING_2 = 'icon-text-heading-2'
    TEXT_HEADING_3 = 'icon-text-heading-3'
    TEXT_HEADING_4 = 'icon-text-heading-4'
    TEXT_HEADING_5 = 'icon-text-heading-5'
    TEXT_HEADING_6 = 'icon-text-heading-6'
    TEXT_HORIZONTALRULE = 'icon-text-horizontalrule'
    TEXT_INDENT = 'icon-text-indent'
    TEXT_INDENT_REMOVE = 'icon-text-indent-remove'
    TEXT_ITALIC = 'icon-text-italic'
    TEXT_KERNING = 'icon-text-kerning'
    TEXT_LETTERSPACING = 'icon-text-letterspacing'
    TEXT_LETTER_OMEGA = 'icon-text-letter-omega'
    TEXT_LINESPACING = 'icon-text-linespacing'
    TEXT_LIST_BULLETS = 'icon-text-list-bullets'
    TEXT_LIST_NUMBERS = 'icon-text-list-numbers'
    TEXT_LOWERCASE = 'icon-text-lowercase'
    TEXT_PADDING_BOTTOM = 'icon-text-padding-bottom'
    TEXT_PADDING_LEFT = 'icon-text-padding-left'
    TEXT_PADDING_RIGHT = 'icon-text-padding-right'
    TEXT_PADDING_TOP = 'icon-text-padding-top'
    TEXT_REPLACE = 'icon-text-replace'
    TEXT_SIGNATURE = 'icon-text-signature'
    TEXT_SMALLCAPS = 'icon-text-smallcaps'
    TEXT_STRIKETHROUGH = 'icon-text-strikethrough'
    TEXT_SUBSCRIPT = 'icon-text-subscript'
    TEXT_SUPERSCRIPT = 'icon-text-superscript'
    TEXT_UNDERLINE = 'icon-text-underline'
    TEXT_UPPERCASE = 'icon-text-uppercase'
    THUMB_DOWN = 'icon-thumb-down'
    THUMB_UP = 'icon-thumb-up'
    TICK = 'icon-tick'
    TIME = 'icon-time'
    TIMELINE_MARKER = 'icon-timeline-marker'
    TIME_ADD = 'icon-time-add'
    TIME_DELETE = 'icon-time-delete'
    TIME_GO = 'icon-time-go'
    TRANSMIT = 'icon-transmit'
    TRANSMIT_ADD = 'icon-transmit-add'
    TRANSMIT_BLUE = 'icon-transmit-blue'
    TRANSMIT_DELETE = 'icon-transmit-delete'
    TRANSMIT_EDIT = 'icon-transmit-edit'
    TRANSMIT_ERROR = 'icon-transmit-error'
    TRANSMIT_GO = 'icon-transmit-go'
    TUX = 'icon-tux'
    USER = 'icon-user'
    USER_ADD = 'icon-user-add'
    USER_COMMENT = 'icon-user-comment'
    USER_DELETE = 'icon-user-delete'
    USER_EDIT = 'icon-user-edit'
    USER_FEMALE = 'icon-user-female'
    USER_GO = 'icon-user-go'
    USER_GRAY = 'icon-user-gray'
    USER_GREEN = 'icon-user-green'
    USER_ORANGE = 'icon-user-orange'
    USER_RED = 'icon-user-red'
    USER_SUIT = 'icon-user-suit'
    VCARD = 'icon-vcard'
    VCARD_ADD = 'icon-vcard-add'
    VCARD_DELETE = 'icon-vcard-delete'
    VCARD_EDIT = 'icon-vcard-edit'
    VECTOR = 'icon-vector'
    VECTOR_ADD = 'icon-vector-add'
    VECTOR_DELETE = 'icon-vector-delete'
    WAND = 'icon-wand'
    WEATHER_CLOUDS = 'icon-weather-clouds'
    WEATHER_CLOUDY = 'icon-weather-cloudy'
    WEATHER_LIGHTNING = 'icon-weather-lightning'
    WEATHER_RAIN = 'icon-weather-rain'
    WEATHER_SNOW = 'icon-weather-snow'
    WEATHER_SUN = 'icon-weather-sun'
    WEBCAM = 'icon-webcam'
    WEBCAM_ADD = 'icon-webcam-add'
    WEBCAM_DELETE = 'icon-webcam-delete'
    WEBCAM_ERROR = 'icon-webcam-error'
    WORLD = 'icon-world'
    WORLD_ADD = 'icon-world-add'
    WORLD_DELETE = 'icon-world-delete'
    WORLD_EDIT = 'icon-world-edit'
    WORLD_GO = 'icon-world-go'
    WORLD_LINK = 'icon-world-link'
    WRENCH = 'icon-wrench'
    WRENCH_ORANGE = 'icon-wrench-orange'
    XHTML = 'icon-xhtml'
    XHTML_ADD = 'icon-xhtml-add'
    XHTML_DELETE = 'icon-xhtml-delete'
    XHTML_GO = 'icon-xhtml-go'
    XHTML_VALID = 'icon-xhtml-valid'
    ZOOM = 'icon-zoom'
    ZOOM_IN = 'icon-zoom-in'
    ZOOM_OUT = 'icon-zoom-out'



#coding: utf-8
"""
Модуль с преднастроенными панелями-деевьями

Created on 25.06.2010

@author: prefer
"""

from m3_ext.ui import containers, controls, menus, render_component
from m3.actions.urls import get_url


class ExtObjectTree(containers.ExtTree):
    """
    Панель с деревом для управления списком объектов.
    """
    #==========================================================================
    # Внутренние классы для ExtObjectTree
    #==========================================================================
    class TreeContextMenu(menus.ExtContextMenu):
        """
        Внутренний класс для удобной работы с контекстным меню дерева
        """
        def __init__(self, *args, **kwargs):
            super(
                ExtObjectTree.TreeContextMenu, self
            ).__init__(
                *args, **kwargs
            )
            self.menuitem_new = menus.ExtContextMenuItem(
                text=u'Новый в корне',
                icon_cls='add_item',
                handler='contextMenuNewRoot'
            )
            self.menuitem_new_child = menus.ExtContextMenuItem(
                text=u'Новый дочерний',
                icon_cls='add_item',
                handler='contextMenuNewChild'
            )
            self.menuitem_edit = menus.ExtContextMenuItem(
                text=u'Изменить',
                icon_cls='edit_item',
                handler='contextMenuEdit'
            )
            self.menuitem_delete = menus.ExtContextMenuItem(
                text=u'Удалить',
                icon_cls='delete_item',
                handler='contextMenuDelete'
            )
            self.menuitem_separator = menus.ExtContextMenuSeparator()
            self.init_component()

    class TreeTopBar(containers.ExtToolBar):
        """
        Внутренний класс для удобной работы топбаром грида
        """
        def __init__(self, *args, **kwargs):
            super(ExtObjectTree.TreeTopBar, self).__init__(*args, **kwargs)

            self.button_new = menus.ExtContextMenuItem(
                text=u'Новый в корне',
                icon_cls='add_item',
                handler='topBarNewRoot'
            )
            self.button_new_child = menus.ExtContextMenuItem(
                text=u'Новый дочерний',
                icon_cls='add_item',
                handler='topBarNewChild'
            )

            self.button_edit = controls.ExtButton(
                text=u'Изменить',
                icon_cls='edit_item',
                handler='topBarEdit'
            )
            self.button_delete = controls.ExtButton(
                text=u'Удалить',
                icon_cls='delete_item',
                handler='topBarDelete'
            )
            self.button_refresh = controls.ExtButton(
                text=u'Обновить',
                icon_cls='refresh-icon-16',
                handler='topBarRefresh'
            )
            menu = menus.ExtContextMenu()
            menu.items.append(self.button_new)
            menu.items.append(self.button_new_child)
            self.add_menu = containers.ExtToolbarMenu(
                icon_cls="add_item",
                menu=menu,
                text=u'Добавить'
            )
            self.init_component()

    #==========================================================================
    # Собственно определение класса ExtObjectTree
    #==========================================================================

    def __init__(self, *args, **kwargs):
        super(ExtObjectTree, self).__init__(*args, **kwargs)
        self.template = 'ext-trees/ext-object-tree.js'
        #======================================================================
        # Действия, выполняемые изнутри грида
        #======================================================================
        self.action_new = None
        self.action_edit = None
        self.action_delete = None
        self.action_data = None

        #======================================================================
        # Источник данных для грида
        #======================================================================
        self.load_mask = True
        self.row_id_name = 'id'
        self.parent_id_name = 'parent_id'
        self.allow_paging = False

        #======================================================================
        # Контекстное меню и бары дерева
        #======================================================================
        self.context_menu_row = ExtObjectTree.TreeContextMenu()
        self.context_menu_tree = ExtObjectTree.TreeContextMenu()
        self.top_bar = ExtObjectTree.TreeTopBar()
        self.top_bar.items.append(self.top_bar.add_menu)
        self.top_bar.items.append(self.top_bar.button_edit)
        self.top_bar.items.append(self.top_bar.button_delete)
        self.top_bar.items.append(self.top_bar.button_refresh)

        self.dblclick_handler = 'onEditRecord'

        # Признак "Сортировки папок"
        # если true, то папки всегда будут выше простых элементов
        # иначе, сортируются как элементы
        self.folder_sort = True

        # После редактирования и добавления обновляется только тот узел дерева,
        # в котором произошли изменения
        self.incremental_update = False

        self.init_component()

    def render(self):
        """
        Переопределяем рендер дерева для того,
        чтобы модифицировать содержимое его
        панелей и контекстных меню
        """
        if self.action_new:
            self.context_menu_row.items.append(
                self.context_menu_row.menuitem_new)
            self.context_menu_row.items.append(
                self.context_menu_row.menuitem_new_child)
            self.context_menu_tree.items.append(
                self.context_menu_tree.menuitem_new)

        if self.action_edit:
            self.context_menu_row.items.append(
                self.context_menu_row.menuitem_edit)
            self.handler_dblclick = self.dblclick_handler

        if self.action_delete:
            self.context_menu_row.items.append(
                self.context_menu_row.menuitem_delete)

        # контекстное меню прицепляется к гриду только в том случае, если
        # в нем есть хотя бы один пункт
        if self.context_menu_tree.items:
            self.handler_contextmenu = self.context_menu_tree
        if self.context_menu_row.items:
            self.handler_rowcontextmenu = self.context_menu_row

        #======================================================================
        # Настройка top bar
        #======================================================================
        for action, btn in (
            (self.action_new, self.top_bar.add_menu),
            (self.action_edit, self.top_bar.button_edit),
            (self.action_delete, self.top_bar.button_delete),
            (self.action_data, self.top_bar.button_refresh),
        ):
            if not action and btn in self.top_bar.items:
                self.top_bar.items.remove(btn)

        # тонкая настройка self.store
        if not self.url and self.action_data:
            self.url = get_url(self.action_data)

        self.render_base_config()
        self.render_params()
        return render_component(self)

    def render_params(self):
        super(ExtObjectTree, self).render_params()

        get_url_or_none = lambda x: get_url(x) if x else None
        new_url = get_url_or_none(self.action_new)
        edit_url = get_url_or_none(self.action_edit)
        delete_url = get_url_or_none(self.action_delete)
        data_url = get_url_or_none(self.action_data)
        context_json = (
            self.action_context.json if self.action_context else None
        )

        self._put_params_value(
            'actions', {
                'newUrl': new_url,
                'editUrl': edit_url,
                'deleteUrl': delete_url,
                'dataUrl': data_url,
                'contextJson': context_json
            }
        )

        self._put_params_value('rowIdName', self.row_id_name)
        self._put_params_value('parentIdName', self.parent_id_name)
        self._put_params_value('folderSort', self.folder_sort)
        self._put_params_value('incrementalUpdate', self.incremental_update)

    def t_render_base_config(self):
        return self._get_config_str()

    def t_render_params(self):
        return self._get_params_str()



#coding:utf-8
"""
Created on 02.03.2010

@author: akvarats
"""

from django.template import TemplateSyntaxError
from m3_ext.ui.containers import ExtForm
from base import BaseExtWindow
from m3_ext.ui.windows.base import ExtWindowRenderer


class ExtEditWindow(BaseExtWindow):
    """Базовый класс окна редактрирования"""

    def __init__(self, *args, **kwargs):
        super(ExtEditWindow, self).__init__(*args, **kwargs)
        self.__form = None
        self.data_url = None  # адрес для загрузки данных формы
        self._ext_name = 'Ext.m3.EditWindow'
        self.renderer = ExtWindowRenderer('ext-script/ext-editwindowscript.js')
        self.init_component(*args, **kwargs)

    @property
    def handler_beforesubmit(self):
        return self._listeners.get('beforesubmit')

    @handler_beforesubmit.setter
    def handler_beforesubmit(self, function):
        self._listeners['beforesubmit'] = function

    @property
    def form(self):
        return self.__form

    @form.setter
    def form(self, value):
        # self.items = [value,] -- Если с этим окном используется всегда форма,
        # то небходимо вставить эту строку
        self.items.append(value)
        self.__form = value

    def render_params(self):
        super(ExtEditWindow, self).render_params()
        if self.form:
            if isinstance(self.form, ExtForm):
                self._put_params_value(
                    'form',
                    {'id': self.form.client_id, 'url': self.form.url})
        self._put_params_value('dataUrl', self.data_url)

    # Данный код должен находится в базовом классе, но т.к. не вcе шаблоны
    # переведены на новый рендеринг, остается пока в каждом
    def render(self):
        assert getattr(self, '_ext_name'), \
            'Class %s is not define "_ext_name"' % (self.__class__.__name__,)

        self.pre_render()

        try:
            self.render_base_config()
            self.render_params()
        except TemplateSyntaxError as msg:
            raise Exception(msg)

        base_config = self._get_config_str()
        params = self._get_params_str()
        res = '%(ext_name)s({%(base_config)s},{%(params)s})' % {
            'ext_name': self._ext_name,
            'base_config': base_config,
            'params': params
        }

        return 'new %s' % res if not self._is_function_render else res
